@@ -57,11 +57,11 @@ void nice_ticks(double *dticks, double *gmin,double *gmax, double *t1,double *tn
 void numtrim(char **d,char *s, double dticks);
 void nice_log_ticks(double *start, double *last, double gmin, double gmax) throw (ParserError);
 
-double fnloglen(double v,axis_struct *ax);
-double fnlogx(double v,axis_struct *ax);
+double fnloglen(double v,GLEAxis *ax);
+double fnlogx(double v,GLEAxis *ax);
 
-void draw_axis_titles(axis_struct *ax, double h, double ox, double oy, GLEMeasureBox* measure);
-void draw_axis_titles_v35(axis_struct *ax, double h, double ox, double oy, double dticks, double llen);
+void draw_axis_titles(GLEAxis *ax, double h, double ox, double oy, GLEMeasureBox* measure);
+void draw_axis_titles_v35(GLEAxis *ax, double h, double ox, double oy, double dticks, double llen);
 
 extern double graph_x1;
 extern double graph_x2;
@@ -75,6 +75,14 @@ extern double graph_y2;
 
 static double  gglen,ggmin,ggmax,ggnegate;
 
+void add_tex_labels(string* title)
+{
+	if (g_get_tex_labels() && str_i_str(*title, "\\tex{") == -1) {
+		title->insert(0, "\\tex{");
+		title->append("}");
+	}
+}
+
 double axis_range_dist_perc(double v1, double v2, GLERange* range, bool log) {
 	if (log) {
 		double lwidth = log10(range->getMax()) - log10(range->getMin());
@@ -124,7 +132,7 @@ bool axis_is_pos_perc(double pos, int* cnt, double perc, vector<double>& vec) {
 	return false;
 }
 
-void axis_draw_tick(axis_struct *ax, double fi, int* tick1_cnt, int* tick2_cnt, double ox, double oy, double t) {
+void axis_draw_tick(GLEAxis *ax, double fi, int* tick1_cnt, int* tick2_cnt, double ox, double oy, double t) {
 	bool has_tick1 = t > 0.0 || ax->ticks_both;
 	bool has_tick2 = t < 0.0 || ax->ticks_both;
 	has_tick1 &= !ax->isNoTick1(fi, tick1_cnt, ax->dsubticks);
@@ -142,7 +150,7 @@ void axis_draw_tick(axis_struct *ax, double fi, int* tick1_cnt, int* tick2_cnt,
 	}
 }
 
-void axis_draw_tick_log(axis_struct *ax, double fi, int* tick1_cnt, int* tick2_cnt, double ox, double oy, double t) {
+void axis_draw_tick_log(GLEAxis *ax, double fi, int* tick1_cnt, int* tick2_cnt, double ox, double oy, double t) {
 	bool has_tick1 = t > 0.0 || ax->ticks_both;
 	bool has_tick2 = t < 0.0 || ax->ticks_both;
 	has_tick1 &= !ax->isNoTick1Perc(fi, tick1_cnt);
@@ -160,7 +168,7 @@ void axis_draw_tick_log(axis_struct *ax, double fi, int* tick1_cnt, int* tick2_c
 	}
 }
 
-void draw_axis(axis_struct *ax, GLERectangle* box, bool drawgrid) {
+void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 	double fi,x,y,gmin,gmax,dticks,tick1,tickn;
 	int i,xax,n,savecap;
 	double tlen,stlen,dsubticks,tt,t;
@@ -464,7 +472,8 @@ void draw_axis(axis_struct *ax, GLERectangle* box, bool drawgrid) {
 						} else if (n == 1) {
 							sprintf(cbuff, "10");
 						} else {
-							sprintf(cbuff, "10\\sup {%d}", n);
+							const char* fmt = g_get_tex_labels() ? "$10^{%d}$" : "10^{%d}";
+							sprintf(cbuff, fmt, n);
 						}
 						ax->setName(i, cbuff);
 					}
@@ -472,8 +481,10 @@ void draw_axis(axis_struct *ax, GLERectangle* box, bool drawgrid) {
 					if (ax->lgset == GLE_AXIS_LOG_25B && format != NULL) {
 						format->format(fi, ax->getNamePtr(i));
 					} else {
-						n = (int) floor(.0001 + fi/(pow(10.0,floor(log10(fi)))));
-						sprintf(cbuff,"{\\sethei{%g}%d}",h*.7,n);
+						n = (int) floor(0.0001 + fi/(pow(10.0,floor(log10(fi)))));
+						double lsize = 0.7 * h * g_get_fconst(GLEC_ALABELSCALE);
+						const char* fmt = g_get_tex_labels() ? "{\\sethei{%g}\\tex{%d}}" : "{\\sethei{%g}%d}";
+						sprintf(cbuff, fmt, lsize, n);
 						ax->setName(i, cbuff);
 					}
 				}
@@ -520,7 +531,9 @@ void draw_axis(axis_struct *ax, GLERectangle* box, bool drawgrid) {
 	if (!ax->label_off && drawrest) {
 		int nb_names = ax->getNbNamedPlaces();
 		for (i = 0; i < nb_names; i++) {
-			g_measure(ax->names[i],&bl,&br,&bu,&bd);
+			string name = ax->names[i];
+			add_tex_labels(&name);
+			g_measure(name, &bl, &br, &bu, &bd);
 			if (bu > maxh) maxh = bu;
 			if (-bd > maxd) maxd = -bd;
 			if (br-bl > maxw) maxw = br-bl;
@@ -529,7 +542,8 @@ void draw_axis(axis_struct *ax, GLERectangle* box, bool drawgrid) {
 		double angle = ax->getLabelAngle();
 		for (i = 0; i < nb_names; i++) {
 			fi = ax->places[i];
-			const string& name = ax->names[i];
+			string name = ax->names[i];
+			add_tex_labels(&name);
 			if (!ax->isNoPlaceLogOrReg(fi, &place_cnt, dticks) && name != "") {
 				fi = fnx(fi);
 				if (ax->log) fi = fnlx(ax->places[i]);
@@ -613,7 +627,7 @@ void draw_axis(axis_struct *ax, GLERectangle* box, bool drawgrid) {
 	g_update_bounds_box(box);
 }
 
-void init_measure_by_axis(axis_struct* ax, double ox, double oy, double llen) {
+void init_measure_by_axis(GLEAxis* ax, double ox, double oy, double llen) {
 	// If there are no labels, use g_set_bounds to initialize "measure"
 	switch (ax->type) {
 		case GLE_AXIS_X:
@@ -634,7 +648,7 @@ void init_measure_by_axis(axis_struct* ax, double ox, double oy, double llen) {
 	}
 }
 
-void draw_axis_titles(axis_struct *ax, double h, double ox, double oy, GLEMeasureBox* measure) {
+void draw_axis_titles(GLEAxis *ax, double h, double ox, double oy, GLEMeasureBox* measure) {
 	double bl,br,bu,bd;
 	g_gsave();
 	double th = h * g_get_fconst(GLEC_ATITLESCALE);
@@ -655,11 +669,8 @@ void draw_axis_titles(axis_struct *ax, double h, double ox, double oy, GLEMeasur
 		ax->alignBase = true;
 	}
 	if (tdist == 0) tdist = h * g_get_fconst(GLEC_ATITLEDIST);
-	string title_str = ax->title;
-	if (g_get_tex_labels()) {
-		title_str.insert(0, "\\tex{");
-		title_str.append("}");
-	}
+	string title_str(ax->title);
+	add_tex_labels(&title_str);
 	g_measure(title_str,&bl,&br,&bu,&bd);
 	switch (ax->type) {
 		case GLE_AXIS_X:
@@ -696,7 +707,7 @@ void draw_axis_titles(axis_struct *ax, double h, double ox, double oy, GLEMeasur
 	g_grestore();
 }
 
-void draw_axis_titles_v35(axis_struct *ax, double h, double ox, double oy, double dticks, double llen) {
+void draw_axis_titles_v35(GLEAxis *ax, double h, double ox, double oy, double dticks, double llen) {
 	double bl,br,bu,bd,x,y;
 	double ty = 0;
 	double tx = ox-.3*h;
@@ -707,13 +718,16 @@ void draw_axis_titles_v35(axis_struct *ax, double h, double ox, double oy, doubl
 	if (!ax->label_off) {
 		int nb_names = ax->getNbNamedPlaces();
 		for (int i = 0; i < nb_names; i++) {
-			g_measure(ax->names[i],&bl,&br,&bu,&bd);
+			string name(ax->names[i]);
+			add_tex_labels(&name);
+			g_measure(name,&bl,&br,&bu,&bd);
 			if (bd>maxd) maxd = bd;
 		}
 		int place_cnt = 0;
 		for (int i = 0; i < nb_names; i++) {
 			double fi = ax->places[i];
-			const string& name = ax->names[i];
+			string name = ax->names[i];
+			add_tex_labels(&name);
 			if (!ax->isNoPlaceLogOrReg(fi, &place_cnt, dticks) && name != "") {
 				fi = fnx(fi);
 				if (ax->log) fi = fnlx(ax->places[i]);
@@ -985,18 +999,18 @@ void numtrim(char **d,char *s,double dticks) {
 	if (nonzero!=NULL) *(nonzero+1) = 0;
 }
 
-double fnloglen(double v,axis_struct *ax) {
+double fnloglen(double v,GLEAxis *ax) {
 	return ((v-log10(ax->getMin()))/(log10(ax->getMax())-log10(ax->getMin()))) * ax->length;
 }
 
-double fnlogx(double v,axis_struct *ax) {
+double fnlogx(double v,GLEAxis *ax) {
 	if (ax->negate) {
 		v = ax->getMax() - (v-ax->getMin());
 	}
 	return fnloglen(log10(v),ax);
 }
 
-void print_axis(axis_struct *ax) {
+void print_axis(GLEAxis *ax) {
 	cout << "Axis ticks off: " << ax->ticks_off << "\tAxis ticks length: "
 	     << ax->ticks_length << "\tAxis ticks color: " << ax->ticks_color << endl;
 	cout << "Axis subticks off: " << ax->subticks_off << "\tAxis subticks length: "
@@ -1085,14 +1099,14 @@ GLEAxisQuantileScale::GLEAxisQuantileScale() {
 GLEAxisQuantileScale::~GLEAxisQuantileScale() {
 }
 
-axis_struct::axis_struct() : format() {
+GLEAxis::GLEAxis() : format() {
 	init(0);
 }
 
-axis_struct::~axis_struct() {
+GLEAxis::~GLEAxis() {
 }
 
-void axis_struct::init(int i) {
+void GLEAxis::init(int i) {
 	int c;
 	format = ""; title = "";
 	clearNoTicks();
@@ -1137,22 +1151,22 @@ void axis_struct::init(int i) {
 	}
 }
 
-string* axis_struct::getNamePtr(int i) {
+string* GLEAxis::getNamePtr(int i) {
 	while ((int)names.size() <= i) names.push_back(string());
 	return &names[i];
 }
 
-void axis_struct::setName(int i, const char* name) {
+void GLEAxis::setName(int i, const char* name) {
 	while ((int)names.size() <= i) names.push_back(string());
 	names[i] = name;
 }
 
-void axis_struct::setPlace(int i, double place) {
+void GLEAxis::setPlace(int i, double place) {
 	while ((int)places.size() <= i) places.push_back(0.0);
 	places[i] = place;
 }
 
-void axis_struct::insertNoTick(double pos, vector<double>& vec) {
+void GLEAxis::insertNoTick(double pos, vector<double>& vec) {
 	vector<double>::size_type at = 0;
 	while (at < vec.size() && vec[at] < pos) {
 		at++;
@@ -1161,33 +1175,33 @@ void axis_struct::insertNoTick(double pos, vector<double>& vec) {
 	else vec.insert(vec.begin()+at, pos);
 }
 
-bool axis_struct::isNoPlaceLogOrReg(double pos, int* place_cnt, double delta) {
+bool GLEAxis::isNoPlaceLogOrReg(double pos, int* place_cnt, double delta) {
 	if (log) return axis_is_pos_perc(pos, place_cnt, 0.001, noplaces);
 	else return axis_is_pos(pos, place_cnt, delta, noplaces);
 }
 
-void axis_struct::addNoTick(double pos) {
+void GLEAxis::addNoTick(double pos) {
 	addNoTick1(pos);
 	addNoTick2(pos);
 }
 
-void axis_struct::insertNoTick(double pos) {
+void GLEAxis::insertNoTick(double pos) {
 	insertNoTick1(pos);
 	insertNoTick2(pos);
 }
 
-void axis_struct::insertNoTickOrLabel(double pos) {
+void GLEAxis::insertNoTickOrLabel(double pos) {
 	insertNoTick1(pos);
 	insertNoTick2(pos);
 	insertNoPlace(pos);
 }
 
-void axis_struct::clearNoTicks() {
+void GLEAxis::clearNoTicks() {
 	noticks1.clear();
 	noticks2.clear();
 }
 
-void axis_struct::printNoTicks() {
+void GLEAxis::printNoTicks() {
 	cout << "Noticks1:";
 	for (vector<double>::size_type i = 0; i < noticks1.size(); i++) {
 		cout << " " << noticks1[i];
@@ -1205,13 +1219,13 @@ void axis_struct::printNoTicks() {
 	cout << endl;
 }
 
-int axis_struct::getNbNamedPlaces() {
+int GLEAxis::getNbNamedPlaces() {
 	int nb = getNbNames();
 	if (getNbPlaces() < nb) nb = getNbPlaces();
 	return nb;
 }
 
-void axis_struct::getLabelsFromDataSet(int ds) {
+void GLEAxis::getLabelsFromDataSet(int ds) {
 	int crpos = 0;
 	if (dp[ds] == NULL) return;
 	int npnts = dp[ds]->np;
@@ -1245,12 +1259,12 @@ void axis_struct::getLabelsFromDataSet(int ds) {
 	}
 }
 
-void axis_struct::initRange() {
+void GLEAxis::initRange() {
 	getDataRange()->initRange();
 	getRange()->initRangeIfNotSet();
 }
 
-void axis_struct::performRoundRange(GLERange* range, bool extend, bool tozero) {
+void GLEAxis::performRoundRange(GLERange* range, bool extend, bool tozero) {
 	if (range->getMax() <= range->getMin()) {
 		return;
 	}
@@ -1264,7 +1278,7 @@ void axis_struct::performRoundRange(GLERange* range, bool extend, bool tozero) {
 	}
 }
 
-void axis_struct::roundDataRange(bool extend, bool tozero) {
+void GLEAxis::roundDataRange(bool extend, bool tozero) {
 	if (getRange()->hasBoth()) {
 		return;
 	}
@@ -1274,7 +1288,7 @@ void axis_struct::roundDataRange(bool extend, bool tozero) {
 	getRange()->copyIfNotSet(drange);
 }
 
-void axis_struct::makeUpRange(axis_struct* copy, axis_struct* orth, bool extend, bool tozero) {
+void GLEAxis::makeUpRange(GLEAxis* copy, GLEAxis* orth, bool extend, bool tozero) {
 	if (getRange()->hasBoth()) {
 		return;
 	}
