@@ -249,6 +249,11 @@ bool CmdLineArgInt::addValue(const string& value) {
 	return true;
 }
 
+void CmdLineArgInt::setValue(int value) {
+	m_Value = value;
+	m_Card++;
+}
+
 void CmdLineArgInt::setDefaultValue() {
 	m_Value = m_Default;
 	m_Card++;
@@ -751,6 +756,11 @@ int CmdLineObj::getIntValue(int option, int arg) {
 	return intval->getValue();
 }
 
+void CmdLineObj::setIntValue(int option, int value, int arg) {
+	CmdLineArgInt* intval = (CmdLineArgInt*)getOption(option)->getArg(arg);
+	intval->setValue(value);
+}
+
 const string& CmdLineObj::getStringValue(int option, int arg) {
 	CmdLineArgString* strval = (CmdLineArgString*)getOption(option)->getArg(arg);
 	return strval->getValue();
