@@ -546,7 +546,7 @@ void complain_about_gletop(bool has_top, ostream& out) {
 	}
 }
 
-bool check_correct_version(const string name, bool has_top, bool has_config, ConfigCollection& coll) {
+bool check_correct_version(const string name, bool has_top, bool has_config, const vector<string>& triedLocations, ConfigCollection& coll) {
 	if (has_config) {
 		const string& val = coll.getStringValue(GLE_CONFIG_GLE, GLE_CONFIG_GLE_VERSION);
 		if (!str_i_equals(val.c_str(), GLEVN)) {
@@ -562,8 +562,11 @@ bool check_correct_version(const string name, bool has_top, bool has_config, Con
 		}
 	} else {
 		ostringstream out;
-		out << "Error: GLE is unable to locate its configuration file:" << endl;
-		out << "       '" << name << "'" << endl;
+		out << "Error: GLE is unable to locate its configuration file." << endl;
+		out << "       GLE searched these locations:" << endl;
+		for (size_t i = 0; i < triedLocations.size(); i++) {
+			out << "       '" << triedLocations[i] << "'" << endl;
+		}
 		complain_about_gletop(has_top, out);
 		g_message(out.str().c_str());
 		return false;
@@ -572,10 +575,10 @@ bool check_correct_version(const string name, bool has_top, bool has_config, Con
 	return true;
 }
 
-bool try_load_config_sub(string& conf_name) {
+bool try_load_config_sub(string& conf_name, vector<string>& triedLocations) {
 	StripDirSep(GLE_TOP_DIR);
 	string conf_name2 = GLE_TOP_DIR + DIR_SEP + "glerc";
-	// cout << "Try: " << conf_name2 << endl;
+	triedLocations.push_back(conf_name2);
 	bool has_config = try_load_config(conf_name2);
 	if (has_config) {
 		// only update name if configuration file found here
@@ -592,6 +595,7 @@ bool do_load_config(const char* appname, char **argv, CmdLineObj& cmdline, Confi
 	bool has_top = false;
 	bool has_config = false;
 	const char* top = getenv("GLE_TOP");
+	vector<string> triedLocations;
 	if (top == NULL || top[0] == 0) {
 		string exe_name;
 		bool has_exe_name = GetExeName(appname, argv, exe_name);
@@ -601,16 +605,16 @@ bool do_load_config(const char* appname, char **argv, CmdLineObj& cmdline, Confi
 			#ifdef GLETOP_CD
 				// Try relative path
 				GLE_TOP_DIR = GLEAddRelPath(exe_name, GLETOP_CD+1, GLETOP_REL);
-				has_config = try_load_config_sub(conf_name);
+				has_config = try_load_config_sub(conf_name, triedLocations);
 				// Try one level higher as executable
 				if (!has_config) {
 					GLE_TOP_DIR = GLEAddRelPath(exe_name, 2, NULL);
-					has_config = try_load_config_sub(conf_name);
+					has_config = try_load_config_sub(conf_name, triedLocations);
 				}
 				// Try with absolute path
 				if (!has_config) {
 					GLE_TOP_DIR = GLETOP_ABS;
-					has_config = try_load_config_sub(conf_name);
+					has_config = try_load_config_sub(conf_name, triedLocations);
 				}
 			#else
 				GLE_TOP_DIR = exe_name;
@@ -632,10 +636,13 @@ bool do_load_config(const char* appname, char **argv, CmdLineObj& cmdline, Confi
 			// Only update conf_name if it was not yet set
 			// To make error message if glerc not found more interpretable
 			conf_name = GLE_TOP_DIR + DIR_SEP + "glerc";
-			has_config = try_load_config(conf_name);
+			if (std::find(triedLocations.begin(), triedLocations.end(), conf_name) == triedLocations.end()) {
+				triedLocations.push_back(conf_name);
+				has_config = try_load_config(conf_name);
+			}
 		}
 	}
-	if (!check_correct_version(conf_name, has_top, has_config, coll)) {
+	if (!check_correct_version(conf_name, has_top, has_config, triedLocations, coll)) {
 		return false;
 	}
 	GLEInterface* iface = GLEGetInterfacePointer();
