@@ -296,6 +296,7 @@ unsigned int farcoreleft();
 
 void g_init();
 void g_get_version(string* version);
+void g_get_build_date(string* date);
 string g_get_version();
 string g_get_version_nosnapshot();
 void g_open(GLEFileLocation* outputfile, const string& inputfile);
@@ -495,6 +496,7 @@ void g_grestore(void);
 void g_gsave(void);
 void g_init_bounds(void);
 void g_jtext(int just);
+void g_line(const GLEPoint& p);
 void g_line(double zx,double zy);
 void g_line_ary(int nwk,double *wkx,double *wky);
 void g_marker_def(char *name, char *subname);
@@ -506,6 +508,7 @@ void g_move(double zx,double zy);
 void g_set_pos(double zx,double zy);
 void g_newpath(void);
 void g_postscript(char *ss,double w,double h) throw (ParserError);
+bool g_parse_ps_boundingbox(const string& line, int* bx1, int* by1, int* bx2, int* by2);
 void g_rdev(double x, double y,double *xd,double *yd);
 void g_reverse(void);
 void g_rline(double zx,double zy);
@@ -535,7 +538,7 @@ void g_set_rgbf(double rr, double gg, double bb, double ff);
 void g_set_rgbf_fill(double rr, double gg, double bb, double ff);
 void g_set_xy(double x, double y);
 void g_stroke(void);
-void g_text(char *s);
+void g_text(const string& s);
 void g_translate(double ztx,double zty);
 void g_undev(double ux,double uy, double *x,double *y);
 void my_char(int ff, int cc);
