@@ -303,7 +303,7 @@ bool str_i_equals(const string& s1, const string& s2) {
 	return true;
 }
 
-int str_starts_with_trim(string& str, const char* test) {
+int str_starts_with_trim(const string& str, const char* test) {
 	int len = str.length();
 	int pos = 0;
 	while (pos < len && (str[pos] == ' ' || str[pos] == '\t')) {
