@@ -48,6 +48,12 @@
 #include <cairo-pdf.h>
 #include <cairo-svg.h>
 
+
+/************************************************************************************
+ * Fonts in Cairo:
+ *
+ ************************************************************************************/
+
 extern struct gmodel g;
 
 char *font_getname(int i);
@@ -101,8 +107,9 @@ void GLECairoDevice::box_fill(dbl x1, dbl y1, dbl x2, dbl y2) {
 	} else {
 		g_flush();
 		cairo_new_path(cr);
+		GLERectangle rect(x1, y1, x2, y2);
 		xdbox(x1,y1,x2,y2);
-		ddfill();
+		ddfill(&rect);
 		cairo_new_path(cr);
 	}
 }
@@ -146,7 +153,8 @@ void GLECairoDevice::circle_fill(double zr) {
 		g_flush();
 		cairo_new_path(cr);
 		cairo_arc(cr, x, y, zr, 0, 2*GLE_PI);
-		ddfill();
+		GLERectangle rect(x-zr, y-zr, x+zr, y+zr);
+		ddfill(&rect);
 		cairo_new_path(cr);
 	}
 }
@@ -166,7 +174,6 @@ void GLECairoDevice::circle_stroke(double zr) {
 }
 
 void GLECairoDevice::clear(void) {
-	//cout << "clear not yet implemented" << endl;
 }
 
 void GLECairoDevice::clip(void) {
@@ -184,7 +191,6 @@ void GLECairoDevice::closepath(void) {
 }
 
 void GLECairoDevice::dfont(char *c) {
-	cout << "dfont not yet implemented" << endl;
 }
 
 void GLECairoDevice::ellipse_fill(double rx, double ry) {
@@ -203,7 +209,8 @@ void GLECairoDevice::ellipse_fill(double rx, double ry) {
 		cairo_scale (cr, rx, ry);
 		cairo_arc (cr, 0, 0, 1, 0, 2*GLE_PI);
 		cairo_restore (cr);
-		ddfill();
+		GLERectangle rect(x-rx, y-ry, x+rx, y+ry);
+		ddfill(&rect);
 		cairo_new_path(cr);
 	}
 }
@@ -268,10 +275,10 @@ void GLECairoDevice::fill(void) {
 	//cairo_restore(cr);
 }
 
-void GLECairoDevice::ddfill(void) {
+void GLECairoDevice::ddfill(GLERectangle* bounds) {
 	if (g_cur_fill.b[B_F] == 255) return; /* clear fill, do nothing */
 	if (g_cur_fill.b[B_F] == 2) {
-		shade();
+		shade(bounds);
 		return;
 	}
 	set_fill();			/*because color and fill are the same*/
@@ -280,144 +287,189 @@ void GLECairoDevice::ddfill(void) {
 	set_color();
 }
 
-void GLECairoDevice::shade(void) {
-	cout << "shade not yet implemented" << endl;
-
+void GLECairoDevice::shadePattern(void) {
 	int step1 = g_cur_fill.b[B_B];
 	int step2 = g_cur_fill.b[B_G];
 	int xstep = max(step1, step2);
 	int ystep = max(step1, step2);
-
-	cout << xstep << endl;
-	cout << ystep << endl;
-
-	cairo_surface_t *isurface;
-	cairo_pattern_t *pattern;
-	cairo_t *icr;
-	cairo_matrix_t   matrix;
-
 	cairo_save(cr);
+	cairo_matrix_t matrix;
 	cairo_get_matrix(cr, &matrix);
-
-	isurface = cairo_surface_create_similar (surface, CAIRO_CONTENT_COLOR_ALPHA, xstep, ystep);
-	icr = cairo_create(isurface);
-
-	//out() << "/BBox [0 0 " << xstep << " " << ystep << "]" << endl;
-	//out() << "/XStep " << xstep << endl;
-	//out() << "/YStep " << ystep << endl;
-	//out() << "/PaintProc" << endl;
-	//out() << "{ pop" << endl;
-	//out() << "1 setgray" << endl;
-	cairo_set_source_rgb(icr, 255.0, 255.0, 255.0);
-	//out() << "2 setlinecap" << endl;
-	cairo_set_line_cap(icr, (cairo_line_cap_t)2);
-	//out() << "-1 -1 " << (xstep+1) << " " << (ystep+1) << " rectfill" << endl;
-	cairo_rectangle(icr, -1, -1, (xstep+1), (ystep+1));
-	cairo_fill(icr);
-
+	cairo_surface_t *isurface = cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, xstep, ystep);
+	cairo_t *icr = cairo_create(isurface);
+	if (m_Background != (int)GLE_FILL_CLEAR) {
+		if (m_Background == (int)GLE_COLOR_WHITE) {
+			cairo_set_source_rgb(icr, 1.0, 1.0, 1.0);
+		} else {
+			colortyp col;
+			col.l = m_Background;
+			cairo_set_source_rgb(icr, col.b[B_R]/255.0, col.b[B_G]/255.0, col.b[B_B]/255.0);
+		}
+		cairo_rectangle(icr, -1, -1, (xstep+1), (ystep+1));
+		cairo_fill(icr);
+	}
 	if (g_cur_fill_color.l == GLE_COLOR_BLACK) {
-		//out() << "0 setgray" << endl;
 		cairo_set_source_rgb(icr, 0, 0, 0);
 	} else {
-		//set_color(g_cur_fill_color);
 		cairo_set_source_rgb(icr, g_cur_fill_color.b[B_R]/255.0, g_cur_fill_color.b[B_G]/255.0, g_cur_fill_color.b[B_B]/255.0);
 	}
-	//out() << (int)g_cur_fill.b[B_R] << " setlinewidth" << endl;
 	cairo_set_line_width(icr, (int)g_cur_fill.b[B_R]);
 	if (step1 > 0) {
-		//out() << "0 0 moveto" << endl;
 		cairo_move_to(icr, 0, 0);
-		//out() << xstep << " " << ystep << " l" << endl;
 		cairo_line_to(icr, xstep, ystep);
-		//out() << "stroke" << endl;
 		cairo_stroke(icr);
 		if (step2 == 0) {
-			//out() << xstep/2 << " " << -ystep/2 << " moveto" << endl;
 			cairo_move_to(icr, xstep/2, -ystep/2);
-			//out() << 3*xstep/2 << " " << ystep/2 << " l" << endl;
 			cairo_line_to(icr, 3*xstep/2, ystep/2);
-			//out() << "stroke" << endl;
 			cairo_stroke(icr);
-			//out() << -xstep/2 << " " << ystep/2 << " moveto" << endl;
 			cairo_move_to(icr, -xstep/2, ystep/2);
-			//out() << xstep/2 << " " << 3*ystep/2 << " l" << endl;
 			cairo_line_to(icr, xstep/2, 3*ystep/2);
-			//out() << "stroke" << endl;
 			cairo_stroke(icr);
 		}
 	}
 	if (step2 > 0) {
-		//out() << "0 " << ystep << " moveto" << endl;
 		cairo_move_to(icr, 0, ystep);
-		//out() << xstep << " 0 l" << endl;
 		cairo_line_to(icr, xstep, 0);
-		//out() << "stroke" << endl;
 		cairo_stroke(icr);
 		if (step1 == 0) {
-			//out() << -xstep/2 << " " << ystep/2 << " moveto" << endl;
 			cairo_move_to(icr, -xstep/2, ystep/2);
-			//out() << xstep/2 << " " << -ystep/2 << " l" << endl;
 			cairo_line_to(icr, xstep/2, -ystep/2);
-			//out() << "stroke" << endl;
 			cairo_stroke(icr);
-			//out() << xstep/2 << " " << 3*ystep/2 << " moveto" << endl;
 			cairo_move_to(icr, xstep/2, 3*ystep/2);
-			//out() << 3*xstep/2 << " " << ystep/2 << " l" << endl;
 			cairo_line_to(icr, 3*xstep/2, ystep/2);
-			//out() << "stroke" << endl;
 			cairo_stroke(icr);
 		}
 	}
 	cairo_set_source_rgb(icr, g_cur_color.b[B_R]/255.0, g_cur_color.b[B_G]/255.0, g_cur_color.b[B_B]/255.0);
-
-	pattern = cairo_pattern_create_for_surface (isurface);
-	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
-
-	cairo_matrix_init_scale (&matrix, 3*72, 3*72);
-	cairo_pattern_set_matrix (pattern, &matrix);
-
-	cairo_set_source (cr, pattern);
-
-	cairo_fill (cr);
+	cairo_pattern_t *pattern = cairo_pattern_create_for_surface(isurface);
+	cairo_pattern_set_extend(pattern, CAIRO_EXTEND_REPEAT);
+	cairo_matrix_init_scale(&matrix, 160.0, 160.0);
+	cairo_pattern_set_matrix(pattern, &matrix);
+	cairo_set_source(cr, pattern);
+	cairo_fill(cr);
 	cairo_restore(cr);
+	cairo_pattern_destroy(pattern);
+	cairo_destroy(icr);
+	cairo_surface_destroy(isurface);
+}
 
-	cairo_pattern_destroy (pattern);
-	cairo_destroy (icr);
-	cairo_surface_destroy (isurface);
+void GLECairoDevice::shadeGLE() {
+	double step1 = g_cur_fill.b[B_B]/160.0;
+	double step2 = g_cur_fill.b[B_G]/160.0;
+	if (step1 > 0) {
+		for (double x = -40.0; x < 40.0; x += step1) {
+			cairo_move_to(cr, x, 0.0);
+			cairo_line_to(cr, 40.0+x, 40);
+			cairo_stroke(cr);
+		}
+	}
+	if (step2 > 0) {
+		for (double x = 0.0; x < 80; x += step2) {
+			cairo_move_to(cr, x, 0.0);
+			cairo_line_to(cr, x-40.0, 40);
+			cairo_stroke(cr);
+		}
+	}
+}
 
-	/*
+void GLECairoDevice::shadeBoundedIfThenElse1(GLERectangle* bounds, double p, double step1) {
+	// if x1+p*s > y1 then
+	if (bounds->getXMax()+p*step1 > bounds->getYMax()) {
+		// aline y1-p*s y1
+		cairo_line_to(cr, bounds->getYMax()-p*step1, bounds->getYMax());
+	} else {
+		// aline x1 x1+p*s
+		cairo_line_to(cr, bounds->getXMax(), bounds->getXMax()+p*step1);
+	}
+	cairo_stroke(cr);
+}
 
-	int              w, h;
-	cairo_surface_t *image;
-	cairo_pattern_t *pattern;
-	cairo_matrix_t   matrix;
+void GLECairoDevice::shadeBoundedIfThenElse2(GLERectangle* bounds, double p, double step2) {
+	// if p*s-y1 > x0 then
+	if (p*step2-bounds->getYMax() > bounds->getXMin()) {
+		// aline p*s-y1 y1
+		cairo_line_to(cr, p*step2-bounds->getYMax(), bounds->getYMax());
+	} else {
+		// aline x0 p*s-x0
+		cairo_line_to(cr, bounds->getXMin(), p*step2-bounds->getXMin());
+	}
+	cairo_stroke(cr);
+}
 
-	image = cairo_image_surface_create_from_png ("test_shade.png");
-	w = cairo_image_surface_get_width (image);
-	h = cairo_image_surface_get_height (image);
+void GLECairoDevice::shadeBounded(GLERectangle* bounds) {
+	double step1 = g_cur_fill.b[B_B]/160.0;
+	double step2 = g_cur_fill.b[B_G]/160.0;
+	cairo_set_line_cap(cr, CAIRO_LINE_CAP_SQUARE);
+	if (step1 > 0) {
+		int p0 = (int)ceil((bounds->getYMax()-bounds->getXMin())/step1-1e-6);
+		if (bounds->getXMin() + p0*step1 > bounds->getYMax()) p0--;
+		int p1 = (int)floor((bounds->getYMin()-bounds->getXMin())/step1+1e-6);
+		if (bounds->getXMin() + p1*step1 < bounds->getYMin()) p1++;
+		int p2 = (int)floor((bounds->getYMin()-bounds->getXMax())/step1+1e-6);
+		if (bounds->getXMax() + p2*step1 < bounds->getYMin()) p2++;
+		for (int p = p0; p > p1; p--) {
+			// amove x0 x0+p*s
+			cairo_move_to(cr, bounds->getXMin(), bounds->getXMin()+p*step1);
+			shadeBoundedIfThenElse1(bounds, p, step1);
+		}
+		for (int p = p1; p >= p2; p--) {
+			// amove y0-p*s y0
+			cairo_move_to(cr, bounds->getYMin()-p*step1, bounds->getYMin());
+			shadeBoundedIfThenElse1(bounds, p, step1);
+		}
+	}
+	if (step2 > 0) {
+		int p0 = (int)ceil((bounds->getYMax()+bounds->getXMax())/step2-1e-6);
+		if (p0*step2 - bounds->getXMin() > bounds->getYMax()) p0--;
+		int p1 = (int)floor((bounds->getYMin()+bounds->getXMax())/step2+1e-6);
+		if (p1*step2 - bounds->getXMax() < bounds->getYMin()) p1++;
+		int p2 = (int)floor((bounds->getYMin()+bounds->getXMin())/step2+1e-6);
+		if (p2*step2 - bounds->getXMax() < bounds->getYMin()) p2++;
+		for (int p = p0; p > p1; p--) {
+			// amove x0 x0+p*s
+			cairo_move_to(cr, bounds->getXMax(), p*step2-bounds->getXMax());
+			shadeBoundedIfThenElse2(bounds, p, step2);
+		}
+		for (int p = p1; p >= p2; p--) {
+			// amove y0-p*s y0
+			cairo_move_to(cr, p*step2-bounds->getYMin(), bounds->getYMin());
+			shadeBoundedIfThenElse2(bounds, p, step2);
+		}
+	}
+}
 
-	pattern = cairo_pattern_create_for_surface (image);
-	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
-
-	cairo_save(cr);
-	//cairo_translate (cr, 128.0, 128.0);
-	//cairo_rotate (cr, M_PI / 4);
-	//cairo_scale (cr, 1 / sqrt (2), 1 / sqrt (2));
-	//cairo_translate (cr, -128.0, -128.0);
-
-
-	//cairo_matrix_init_scale (&matrix, w/256.0 * 5.0, h/256.0 * 5.0);
-	cairo_get_matrix(cr, &matrix);
-	cairo_pattern_set_matrix (pattern, &matrix);
-
-	cairo_set_source (cr, pattern);
-
-	//cairo_rectangle (cr, 0, 0, 256.0, 256.0);
-	cairo_fill_preserve (cr);
-	cairo_restore(cr);
-	cairo_pattern_destroy (pattern);
-	cairo_surface_destroy (image);
-*/
+void GLECairoDevice::shade(GLERectangle* bounds) {
+	if (m_FillMethod == GLE_FILL_METHOD_GLE ||
+	   (m_FillMethod == GLE_FILL_METHOD_DEFAULT && bounds != NULL)) {
+		cairo_save(cr);
+		if (m_Background != (int)GLE_FILL_CLEAR) {
+			if (m_Background == (int)GLE_COLOR_WHITE) {
+				cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+			} else {
+				colortyp col;
+				col.l = m_Background;
+				set_color(col);
+			}
+			cairo_fill_preserve(cr);
+		}
+		// Implemented by using path as clip and then painting strokes over the clip
+		cairo_clip(cr);
+		cairo_new_path(cr);
+		if (g_cur_fill_color.l == GLE_COLOR_BLACK) {
+			cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+		} else {
+			set_color(g_cur_fill_color);
+		}
+		cairo_set_line_width(cr, (double)(g_cur_fill.b[B_R]/160.0));
+		if (m_FillMethod == GLE_FILL_METHOD_DEFAULT && bounds != NULL) {
+			shadeBounded(bounds);
+		} else {
+			shadeGLE();
+		}
+		cairo_restore(cr);
+	} else {
+		shadePattern();
+	}
 }
 
 void GLECairoDevice::fill_ary(int nwk,double *wkx,double *wky) {
@@ -448,7 +500,7 @@ void GLECairoDevice::line_ary(int nwk,double *wkx,double *wky) {
 }
 
 void GLECairoDevice::message(char *s) {
-	cout << "message not yet implemented" << endl;
+	printf("%s\n",s);
 }
 
 void GLECairoDevice::move(double zx,double zy) {
@@ -506,8 +558,16 @@ void GLECairoDevice::set_fill(int f) {
 	g_cur_fill.l = f;
 }
 
+void GLECairoDevice::set_background(int b) {
+	m_Background = b;
+}
+
+void GLECairoDevice::set_fill_method(int m) {
+	m_FillMethod = m;
+}
+
 void GLECairoDevice::set_pattern_color(int c) {
-	cout << "set_pattern_color not yet implemented" << endl;
+	g_cur_fill_color.l = c;
 }
 
 void GLECairoDevice::set_line_cap(int i) {
@@ -545,7 +605,8 @@ void GLECairoDevice::set_line_style(const char *s) {
 }
 
 void GLECairoDevice::set_line_styled(double dd) {
-	//cout << "set_line_styled not yet implemented" << endl;
+	// no need to implement: set_line_style gets this from context
+	// FIXME: rather: should call set_line_style again?
 }
 
 void GLECairoDevice::set_line_width(double w) {
@@ -556,14 +617,14 @@ void GLECairoDevice::set_line_width(double w) {
 }
 
 void GLECairoDevice::set_matrix(double newmat[3][3]) {
-    cairo_matrix_t matrix;
-    matrix.xx = newmat[0][0];
-    matrix.xy = newmat[0][1];
-    matrix.yx = - newmat[1][0];
-    matrix.yy = - newmat[1][1];
-    matrix.x0 = newmat[0][2];
-    matrix.y0 = m_height * 72 / CM_PER_INCH - newmat[1][2];
-    cairo_set_matrix(cr, &matrix);
+	cairo_matrix_t matrix;
+	matrix.xx = newmat[0][0];
+	matrix.xy = newmat[0][1];
+	matrix.yx = - newmat[1][0];
+	matrix.yy = - newmat[1][1];
+	matrix.x0 = newmat[0][2];
+	matrix.y0 = m_height * 72 / CM_PER_INCH - newmat[1][2];
+	cairo_set_matrix(cr, &matrix);
 }
 
 void GLECairoDevice::set_path(int onoff) {
@@ -575,9 +636,6 @@ void GLECairoDevice::source(const char *s) {
 }
 
 void GLECairoDevice::stroke(void) {
-	//cairo_save(cr);
-	//cairo_stroke(cr);
-	//cairo_restore(cr);
 	cairo_stroke_preserve(cr);
 }
 
@@ -618,10 +676,10 @@ void GLECairoDeviceSVG::opendev(double width, double height, GLEFileLocation* ou
 	m_height = height;
 	m_OutputName.copy(outputfile);
 	m_OutputName.addExtension("svg");
-    surface = cairo_svg_surface_create(m_OutputName.getFullPath().c_str(), 72*width/CM_PER_INCH+2, 72*height/CM_PER_INCH+2);
+	surface = cairo_svg_surface_create(m_OutputName.getFullPath().c_str(), 72*width/CM_PER_INCH+2, 72*height/CM_PER_INCH+2);
 	cr = cairo_create(surface);
 	g_scale(72.0/CM_PER_INCH, 72.0/CM_PER_INCH);
-	g_translate(1.0*CM_PER_INCH/72, 1.0*CM_PER_INCH/72);
+	g_translate(1.0*CM_PER_INCH/72, -1.0*CM_PER_INCH/72);
 }
 
 int GLECairoDeviceSVG::getDeviceType() {
@@ -642,11 +700,141 @@ void GLECairoDevicePDF::opendev(double width, double height, GLEFileLocation* ou
 	surface = cairo_pdf_surface_create(m_OutputName.getFullPath().c_str(), 72*width/CM_PER_INCH+2, 72*height/CM_PER_INCH+2);
 	cr = cairo_create(surface);
 	g_scale(72.0/CM_PER_INCH, 72.0/CM_PER_INCH);
-	g_translate(1.0*CM_PER_INCH/72, 1.0*CM_PER_INCH/72);
+	g_translate(1.0*CM_PER_INCH/72, -1.0*CM_PER_INCH/72);
 }
 
 int GLECairoDevicePDF::getDeviceType() {
 	return GLE_DEVICE_CAIRO_PDF;
 }
 
+#ifdef __WIN32__
+
+#include <cairo-win32.h>
+
+class GLECairoDeviceEMFWinInfo {
+public:
+	HDC hdc;
+	bool shouldClose;
+	GLECairoDeviceEMFWinInfo();
+	~GLECairoDeviceEMFWinInfo();
+};
+
+GLECairoDeviceEMFWinInfo::GLECairoDeviceEMFWinInfo() :
+	shouldClose(false)
+{
+}
+
+GLECairoDeviceEMFWinInfo::~GLECairoDeviceEMFWinInfo()
+{
+	if (shouldClose) {
+		HENHMETAFILE meta = CloseEnhMetaFile(hdc);
+		DeleteEnhMetaFile(meta);
+	}
+}
+
+GLECairoDeviceEMF::GLECairoDeviceEMF(bool showerror) : GLECairoDevice(showerror) {
+	m_WinInfo = new GLECairoDeviceEMFWinInfo();
+	m_DPI = 1000;
+	m_CopyClipboard = false;
+}
+
+GLECairoDeviceEMF::~GLECairoDeviceEMF() {
+	delete m_WinInfo;
+}
+
+void GLECairoDeviceEMF::set_matrix(double newmat[3][3]) {
+    cairo_matrix_t matrix;
+    matrix.xx = newmat[0][0];
+    matrix.xy = newmat[0][1];
+    matrix.yx = - newmat[1][0];
+    matrix.yy = - newmat[1][1];
+    matrix.x0 = newmat[0][2];
+    matrix.y0 = m_height * m_DPI/CM_PER_INCH - newmat[1][2];
+    cairo_set_matrix(cr, &matrix);
+}
+
+void GLECairoDeviceEMF::opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError) {
+	m_width = width;
+	m_height = height;
+	m_OutputName.copy(outputfile);
+	m_OutputName.addExtension("emf");
+	double myWidth = width + 2.0/72.0*CM_PER_INCH;
+	double myHeight = height + 2.0/72.0*CM_PER_INCH;
+	HDC hdcRef = GetDC(NULL);
+	int iWidthMM = GetDeviceCaps(hdcRef, HORZSIZE);    // iWidthMM is the display width in millimeters.
+	int iHeightMM = GetDeviceCaps(hdcRef, VERTSIZE);   // iHeightMM is the display height in millimeters.
+	int iWidthPels = GetDeviceCaps(hdcRef, HORZRES);   // iWidthPels is the display width in pixels.
+	int iHeightPels = GetDeviceCaps(hdcRef, VERTRES);  // iHeightPels is the display height in pixels.
+	RECT dim;
+	dim.left = 0;
+	dim.top = 0;
+	dim.right = (int)floor(1000.0 * myWidth + 0.5);
+	dim.bottom = (int)floor(1000.0 * myHeight + 0.5);
+	// cout << "dim.right = " << dim.right << endl;
+	// cout << "dim.bottom = " << dim.bottom << endl;
+	if (m_CopyClipboard) {
+		m_WinInfo->hdc = CreateEnhMetaFile(hdcRef, NULL, &dim, NULL);
+	} else {
+		m_WinInfo->hdc = CreateEnhMetaFile(hdcRef, m_OutputName.getFullPath().c_str(), &dim, NULL);
+	}
+	ReleaseDC(NULL, hdcRef);
+	if (m_WinInfo->hdc != NULL) {
+		m_WinInfo->shouldClose = true;
+	} else {
+		g_throw_parser_error("error creating EMF file: '", m_OutputName.getFullPath().c_str(), "'");
+	}
+	// cout << "Width = " << iWidthMM << " mm = " << iWidthPels << " pixels" << endl;
+	// cout << "Height = " << iHeightMM << " mm = " << iHeightPels << " pixels" << endl;
+	int viewportWidth = (int)floor(10.0 * myWidth * iWidthPels / iWidthMM + 0.5);
+	int viewportHeight = (int)floor(10.0 * myHeight * iHeightPels / iHeightMM + 0.5);
+	// cout << "Viewport width in pixels = " << viewportWidth << endl;
+	// cout << "Viewport height in pixels = " << viewportHeight << endl;
+	int windowWidth = (int)floor(myWidth * m_DPI / CM_PER_INCH + 0.5);
+	int windowHeight = (int)floor(myHeight * m_DPI / CM_PER_INCH + 0.5);
+	// cout << "Window width in pixels = " << windowWidth << endl;
+	// cout << "Window height in pixels = " << windowHeight << endl;
+	SetMapMode(m_WinInfo->hdc, MM_ANISOTROPIC);
+	SetWindowOrgEx(m_WinInfo->hdc, 0, 0, NULL);
+	SetWindowExtEx(m_WinInfo->hdc, windowWidth, windowHeight, NULL);
+	SetViewportExtEx(m_WinInfo->hdc, viewportWidth, viewportHeight, NULL);
+	surface = cairo_win32_printing_surface_create(m_WinInfo->hdc);
+	cr = cairo_create(surface);
+	g_scale(m_DPI/CM_PER_INCH, m_DPI/CM_PER_INCH);
+	g_translate(1.0*CM_PER_INCH/72, 1.0*CM_PER_INCH/72);
+}
+
+BOOL PutEnhMetafileOnClipboard(HWND hWnd, HENHMETAFILE hEMF) {
+	BOOL bResult = false;
+	HENHMETAFILE hEMF2 = CopyEnhMetaFile(hEMF, NULL);
+	if (hEMF2 != NULL) {
+		if (OpenClipboard(hWnd)) {
+			if (EmptyClipboard()) {
+				HANDLE hRes = (HENHMETAFILE)SetClipboardData(CF_ENHMETAFILE, hEMF2);
+				bResult = (hRes != NULL);
+			}
+			CloseClipboard();
+		}
+	}
+	return bResult;
+}
+
+void GLECairoDeviceEMF::closedev(void) {
+	GLECairoDevice::closedev();
+	HENHMETAFILE meta = CloseEnhMetaFile(m_WinInfo->hdc);
+	if (meta == NULL) {
+		return;
+	}
+	if (m_CopyClipboard) {
+		PutEnhMetafileOnClipboard(NULL, meta);
+	}
+	DeleteEnhMetaFile(meta);
+	m_WinInfo->shouldClose = false;
+}
+
+int GLECairoDeviceEMF::getDeviceType() {
+	return GLE_DEVICE_EMF;
+}
+
+#endif
+
 #endif
