@@ -107,9 +107,9 @@ public:
 	virtual FILE* get_file_pointer(void) = 0;
 	virtual int getDeviceType() = 0;
 	virtual void bitmap(GLEBitmap* bitmap, GLEPoint* pos, GLEPoint* scale, int type);
-	virtual void writeRecordedOutput(ostream& out) throw (ParserError);
 	virtual const char* getExtension();
-	void writeRecordedOutputFile(const string& fname) throw (ParserError);
+	virtual void getRecordedBytes(string* output);
+	void writeRecordedOutputFile(const string& fname, string* buffer = 0) throw (ParserError);
 public:
 	inline void setRecordingEnabled(bool rec) { m_Recording = rec; }
 	inline bool isRecordingEnabled() { return m_Recording; }
@@ -118,7 +118,7 @@ public:
 class PSGLEDevice : public GLEDevice {
 protected:
 	GLEFileLocation m_OutputName;
-	stringstream* m_OutputBuffer;
+	ostringstream* m_OutputBuffer;
 	ofstream* m_OutputFile;
 	ostream* m_Out;
 	bool m_IsEps;
@@ -192,7 +192,7 @@ public:
 	virtual int getDeviceType();
 	virtual void bitmap(GLEBitmap* bitmap, GLEPoint* pos, GLEPoint* scale, int type);
 	virtual const char* getExtension();
-	virtual void writeRecordedOutput(ostream& out) throw (ParserError);
+	virtual void getRecordedBytes(string* output);
 protected:
 	void set_color(colortyp& color);
 	void ddfill(GLERectangle* bounds = NULL);
@@ -212,10 +212,7 @@ public:
 	inline GLEPoint* getBoundingBox() { return &m_BBox; }
 	inline ostream& out() { return *m_Out; }
 	void startRecording();
-	void getRecordedPostScript(string* result);
 	void initialPS();
-public:
-	inline stringstream* getRecordedPostScript() { return m_OutputBuffer; }
 };
 
 class SVGGLEDevice : public GLEDevice {
@@ -307,8 +304,10 @@ protected:
 	bool m_ShowError;
 	double m_width;
 	double m_height;
-    cairo_surface_t *surface;
-    cairo_t *cr;
+	cairo_surface_t *surface;
+	cairo_t *cr;
+	int m_FillMethod;
+	int m_Background;
 	colortyp g_cur_fill, g_cur_color, g_cur_fill_color;
 public:
 	GLECairoDevice(bool showerror);
@@ -348,6 +347,8 @@ public:
 	virtual void reverse(void)    /* reverse the order of stuff in the current path */;
 	virtual void set_color(int f);
 	virtual void set_fill(int f);
+	virtual void set_background(int b);
+	virtual void set_fill_method(int m);
 	virtual void set_pattern_color(int c);
 	virtual void set_line_cap(int i);
 	virtual void set_line_join(int i);
@@ -367,8 +368,13 @@ public:
 	virtual int getDeviceType();
 protected:
 	void set_color(colortyp& color);
-	void ddfill(void);
-	void shade(void);
+	void ddfill(GLERectangle* bounds = NULL);
+	void shadePattern();
+	void shadeGLE();
+	void shadeBoundedIfThenElse1(GLERectangle* bounds, double p, double step1);
+	void shadeBoundedIfThenElse2(GLERectangle* bounds, double p, double step2);
+	void shadeBounded(GLERectangle* bounds);
+	void shade(GLERectangle* bounds);
 };
 
 class GLECairoDevicePDF : public GLECairoDevice {
@@ -387,6 +393,28 @@ public:
 	virtual int getDeviceType();
 };
 
+#ifdef __WIN32__
+
+class GLECairoDeviceEMFWinInfo;
+
+class GLECairoDeviceEMF : public GLECairoDevice {
+protected:
+	GLECairoDeviceEMFWinInfo* m_WinInfo;
+	double m_DPI;
+	bool m_CopyClipboard;
+public:
+	GLECairoDeviceEMF(bool showerror);
+	virtual ~GLECairoDeviceEMF();
+	virtual void set_matrix(double newmat[3][3]);
+	virtual void opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError);
+	virtual int getDeviceType();
+	virtual void closedev(void);
+	inline void setDPI(double d) { m_DPI = d; }
+	inline void setCopyClipboard(bool c) { m_CopyClipboard = true; }
+};
+
+#endif
+
 #endif
 
 class GLEDummyDevice : public GLEDevice {
