@@ -339,7 +339,13 @@ void PSGLEDevice::set_line_style(const char *s) {
 	if (!g.inpath) g_flush();
 	strcpy(ob,"[");
 	if (strlen(s) == 1) {
-		s = defline[*s-'0'];
+		int index = s[0] - '0';
+		if (index < 0 || index > (int)(sizeof(defline)/sizeof(const char*))) {
+			ostringstream err;
+			err << "illegal line style '" << s << "'";
+			g_throw_parser_error(err.str());
+		}
+		s = defline[index];
 	}
 	int l = strlen(s);
 	for (i=0;i<l;i++) {
@@ -1114,15 +1120,10 @@ void PSGLEDevice::startRecording() {
 	if (m_OutputBuffer != NULL) {
 		delete m_OutputBuffer;
 	}
-	m_OutputBuffer = new stringstream();
+	m_OutputBuffer = new ostringstream();
 	m_Out = m_OutputBuffer;
 }
 
-void PSGLEDevice::getRecordedPostScript(string* result) {
+void PSGLEDevice::getRecordedBytes(string* result) {
 	*result = m_OutputBuffer->str();
 }
-
-void PSGLEDevice::writeRecordedOutput(ostream& out) throw (ParserError) {
-	m_OutputBuffer->rdbuf()->pubseekpos(0);
-	GLECopyStream(*m_OutputBuffer, out);
-}
