@@ -55,6 +55,7 @@
 #include "cmdline.h"
 #include "graph.h"
 #include "sub.h"
+#include "file_io.h"
 #ifdef __WIN32__
 #include <time.h>
 #endif
@@ -499,6 +500,14 @@ void eval_pcode_loop(int *pcode, int plen, int *otyp) throw(ParserError) {
 			if (stk_str[nstk] != NULL) myfree(stk_str[nstk]);
 			stk_str[nstk] = sdup(sbuf);
 			break;
+		case 60+FN_GETENV: /* getenv */
+			*otyp = 2;
+			{
+				string result;
+				GLEGetEnv(string(stk_str[nstk]), result);
+				stk_str[nstk] = sdup(result.c_str());
+			}
+			break;
 		case 66: /* date$ */
 			{
 				*otyp = 2;
