@@ -118,6 +118,17 @@ using namespace std;
 string GLE_TOP_DIR;
 string GLE_BIN_DIR;
 
+bool GLEGetEnv(const string& name, string& result) {
+	const char* env = getenv(name.c_str());
+	if (env == NULL) {
+		result = "";
+		return false;
+	} else {
+		result = env;
+		return true;
+	}
+}
+
 //
 // -- directory finding routines
 //
@@ -965,7 +976,7 @@ bool DeleteFileWithNewExt(const string& fname, const char* ext) {
 
 // Delete file with extension replaced by given one
 bool DeleteFileWithExt(const string& fname, const char* ext) {
-	string name = fname;
+	string name(fname);
 	name += ext;
 	return TryDeleteFile(name);
 }
@@ -1504,4 +1515,18 @@ string GLEExpandEnvironmentVariables(const string& str) {
 	return result.str();
 }
 
+bool GLEReadFile(const string& name, vector<string>* lines) {
+	ifstream inFile(name.c_str());
+	if (!inFile.is_open())
+	{
+		return false;
+	}
+	while (inFile.good()) {
+		string line;
+		getline(inFile, line);
+		lines->push_back(line);
+	}
+	inFile.close();
+	return true;
+}
 
