@@ -87,6 +87,7 @@ public:
 string fontdir(const char *fname);
 char *line(int i);
 
+bool GLEGetEnv(const string& name, string& result);
 void CopyGLETop(char* buf);
 string GetActualFilename(ifstream* file, const string& fname, const string* directory);
 void FillIncludePaths(vector<string>& IP);
@@ -140,6 +141,7 @@ void StripPathComponents(string* fname, int nb);
 bool IsAbsPath(const string& path);
 void GLEGetFullPath(const string& dirname, const string& fname, string& fullpath);
 string GLEExpandEnvironmentVariables(const string& str);
+bool GLEReadFile(const string& name, vector<string>* lines);
 
 #define GLE_SYSTEM_OK            0
 #define GLE_SYSTEM_ERROR         1
