@@ -289,6 +289,17 @@ void GLEString::toUTF8(string& out) const {
 	}
 }
 
+string GLEString::toUTF8() const {
+	string out;
+	GLEStringToUTF8 conv(this);
+	while (1) {
+		char ch = conv.get();
+		if (ch == 0) break;
+		out.push_back(ch);
+	}
+	return out;
+}
+
 void GLEString::toUTF8(char* out) const {
 	int len = 0;
 	GLEStringToUTF8 conv(this);
@@ -440,6 +451,11 @@ int GLEString::getType() {
 	return GLEObjectTypeString;
 }
 
+GLEString* GLEString::getEmptyString() {
+	static GLERC<GLEString> g_EmptyString = new GLEString();
+	return g_EmptyString.get();
+}
+
 GLEArray::GLEArray() {
 }
 
