@@ -214,6 +214,12 @@ string GLEInterface::getGLEVersion() {
 	return version;
 }
 
+string GLEInterface::getGLEBuildDate() {
+	string date;
+	g_get_build_date(&date);
+	return date;
+}
+
 const char* GLEInterface::getGLETop() {
 	return GLE_TOP_DIR.c_str();
 }
@@ -292,14 +298,18 @@ void GLEInterface::setCompatibilityMode(const char* version) {
 	g_set_compatibility(version);
 }
 
-void GLEInterface::renderGLE(GLEScript* script, const char* outfile, int device) {
+void GLEInterface::renderGLE(GLEScript* script, const char* outfile, int device, bool toMemory) {
 	m_Script = script;
 	if (script == NULL) {
 		cerr << "GLEInterface::renderGLE(): script == NULL" << endl;
 		return;
 	}
+	script->cleanUp();
 	try {
 		size_t exit_code;
+		// Select dry-run (and output to memory)
+		CmdLineOption* noSave = g_CmdLine.createOption(GLE_OPT_NOSAVE);
+		noSave->setHasOption(toMemory);
 		// Set the device
 		CmdLineArgSet* dev = (CmdLineArgSet*)g_CmdLine.createOption(GLE_OPT_DEVICE)->getArg(0);
 		dev->reset();
@@ -460,7 +470,7 @@ const char* GLEInterface::getInitialPostScript() {
 		dev->startRecording();
 		dev->initialPS();
 		m_InitialPS = new string();
-		dev->getRecordedPostScript(m_InitialPS);
+		dev->getRecordedBytes(m_InitialPS);
 		saved_state.restore();
 	}
 	return m_InitialPS->c_str();
@@ -517,7 +527,7 @@ void GLEInterface::renderText(GLETextDO* text, GLEPropertyStore* prop) {
 	g_move(0.0, 0.0);
 	g_jtext(JUST_BL);
 	// Return the resulting PostScript code
-	dev->getRecordedPostScript(text->getPostScriptPtr());
+	dev->getRecordedBytes(text->getPostScriptPtr());
 /*
 	string code;
 	code += getInitialPostScript();
@@ -534,12 +544,38 @@ GLETextDO* GLEInterface::renderText(const char* str, GLEPropertyStore* prop) {
 	return text;
 }
 
+bool GLEInterface::isDeviceSupported(int device) {
+	switch (device) {
+		case GLE_DEVICE_PS:
+		case GLE_DEVICE_EPS:
+		case GLE_DEVICE_DUMMY:
+			return true;
+#ifdef HAVE_CAIRO
+        case GLE_DEVICE_CAIRO_PDF:
+		case GLE_DEVICE_SVG:
+        case GLE_DEVICE_CAIRO_SVG:
+			return true;
+#ifdef __WIN32__
+        case GLE_DEVICE_EMF:
+			return true;
+#endif
+#endif
+#ifdef HAVE_X11
+		case GLE_DEVICE_X11:
+			return true;
+#endif
+		default:
+			return false;
+	}
+}
+
 const char* GLEInterface::getDeviceFilenameExtension(int device) {
 	switch (device) {
 		case GLE_DEVICE_PDF:  return "pdf";
 		case GLE_DEVICE_PS:   return "ps";
 		case GLE_DEVICE_EPS:  return "eps";
 		case GLE_DEVICE_SVG:  return "svg";
+		case GLE_DEVICE_EMF:  return "emf";
 		case GLE_DEVICE_PNG:  return "png";
 		case GLE_DEVICE_JPEG: return "jpg";
 	}
@@ -1026,6 +1062,7 @@ void GLEScript::cleanUp() {
 	m_Parser = NULL;
 	m_Polish = NULL;
 	m_Pcode = NULL;
+	m_PostScript.resize(0);
 }
 
 GLEDrawObject* GLEScript::newGLEObject(GLEDrawObjectType type) {
@@ -1107,6 +1144,16 @@ void GLEScript::updateObjectDOConstructors() {
 	}
 }
 
+const char* GLEScript::getPostScriptCode()
+{
+	return m_PostScript.c_str();
+}
+
+string* GLEScript::getRecordedBytesBuffer(int /* device */)
+{
+	return &m_PostScript;
+}
+
 GLELineDO::GLELineDO() {
 	m_Arrow = GLEHasArrowNone;
 }
@@ -1414,6 +1461,10 @@ void GLETextDO::createGLECode(string& code) {
 }
 
 void GLETextDO::updateBoundingBox() {
+	GLEDevice* oldDevice = g_set_dummy_device();
+	g_move(getPosition());
+	g_text(getText());
+	g_restore_device(oldDevice);
 	g_move(getPosition());
 }
 
@@ -1643,7 +1694,7 @@ void GLEObjectDO::render() {
 		// cout << "Dev. Rectangle: " << *newobj->getRectangle() << endl;
 		run->setCRObjectRep(NULL);
 		// Return the resulting PostScript code
-		dev->getRecordedPostScript(getPostScriptPtr());
+		dev->getRecordedBytes(getPostScriptPtr());
 		saved_state.restore();
 	} catch (ParserError err) {
 		newobj->getRectangle()->setXMin(-1.0);
