@@ -148,6 +148,8 @@ public:
 	bool isMinValid() const;
 	bool isMaxValid() const;
 	void updateRange(double value);
+	void clip(double* value);
+	bool contains(double value);
 	ostream& printRange(ostream& out) const;
 	inline double getMin() { return m_Min; }
 	inline double getMax() { return m_Max; }
@@ -269,6 +271,54 @@ inline ostream& operator<<(ostream& os, const GLEPoint& pt) {
 	return pt.write(os);
 }
 
+class DLLCLASS GLEPoint3D {
+public:
+	double m_C[3];
+public:
+	GLEPoint3D();
+	GLEPoint3D(double x, double y, double z);
+	GLEPoint3D(const GLEPoint3D& p);
+	inline double get(int i) const { return m_C[i]; }
+	inline void set(int i, double v) { m_C[i] = v; }
+	inline void set(double x, double y, double z) { m_C[0] = x; m_C[1] = y; m_C[2] = z; }
+	DLLFCT void dotScalar(double s);
+	DLLFCT void add(const GLEPoint3D& p);
+	DLLFCT void subtract(const GLEPoint3D& p);
+	DLLFCT void addScalar(double s1, double s2, const GLEPoint3D& p);
+	DLLFCT double norm() const;
+	DLLFCT void normalize();
+	void ortho3DUnit(const GLEPoint3D& p, GLEPoint3D* r);
+	DLLFCT ostream& write(ostream& os) const;
+};
+
+class DLLCLASS GLEMatrix {
+public:
+	double* m_C;
+	int m_Rows, m_Cols;
+public:
+	GLEMatrix(int rows, int cols);
+	GLEMatrix(const GLEMatrix& p);
+	~GLEMatrix();
+	void dot(const GLEPoint3D& p, GLEPoint3D* r) const;
+	void setVertVector(int row, int col, const GLEPoint3D& p);
+	DLLFCT ostream& write(ostream& os) const;
+};
+
+class DLLCLASS GLEProjection {
+public:
+	GLEPoint3D m_Eye, m_Reference, m_VVector;
+public:
+	GLEProjection();
+	inline GLEPoint3D* getEye() { return &m_Eye; }
+	inline GLEPoint3D* getReference() { return &m_Reference; }
+	inline GLEPoint3D* getV() { return &m_VVector; }
+	void invToReference(GLEMatrix* mtrx);
+	DLLFCT void zoom(double factor);
+	DLLFCT void rotate(double angle, bool horiz);
+	DLLFCT void reference(const GLEPoint3D& ref);
+	DLLFCT void adjustV(double angle);
+};
+
 class DLLCLASS GLELinearEquation {
 protected:
 	double m_A, m_B;
@@ -276,6 +326,7 @@ public:
 	GLELinearEquation();
 	~GLELinearEquation();
 	void DLLFCT fit(double x0, double y0, double x1, double y1);
+	inline double intersect(GLELinearEquation* o) { return (o->m_B - m_B)/(m_A - o->m_A); }
 	inline void fitB(double x0, double y0) { m_B = y0 - m_A*x0; }
 	inline void setA(double a) { m_A = a; }
 	inline void setB(double b) { m_B = b; }
@@ -611,6 +662,8 @@ public:
 	inline void setFullPath(const string& v) { m_FullPath = v; }
 };
 
+struct surface_struct;
+
 class DLLCLASS GLEInterface {
 protected:
 	GLEScript* m_Script;
@@ -637,6 +690,7 @@ public:
 
 	// Returns the version of GLE
 	DLLFCT string getGLEVersion();
+	DLLFCT string getGLEBuildDate();
 
 	// Returns the location of GLE_TOP
 	DLLFCT const char* getGLETop();
@@ -673,7 +727,7 @@ public:
 
 	DLLFCT void setCompatibilityMode(const char* version);
 
-	DLLFCT void renderGLE(GLEScript* script, const char* outfile, int device);
+	DLLFCT void renderGLE(GLEScript* script, const char* outfile, int device, bool toMemory = false);
 
 	DLLFCT void commitChangesGLE(GLEScript* script);
 
@@ -715,6 +769,8 @@ public:
 
 	inline GLEOutputStream* getOutput() { return m_Output; }
 
+	DLLFCT bool isDeviceSupported(int device);
+
 	DLLFCT const char* getDeviceFilenameExtension(int device);
 
 	// Return the property store model for a line
@@ -745,6 +801,9 @@ public:
 	DLLFCT int copyFile(const string& from, const string& to, string* err = NULL);
 
 	void initTextProperties(GLEPropertyStore* prop);
+
+	// Get information about 3D surface plots
+	DLLFCT surface_struct* getSurface();
 public:
 	inline void setCommitMode(bool commit) { m_CommitMode = commit; }
 	inline bool isCommitMode() { return m_CommitMode; }
@@ -1058,8 +1117,9 @@ protected:
 	GLEParser* m_Parser;
 	GLEPolish* m_Polish;
 	GLEGlobalSource m_File;
-	GLEPoint m_Size, m_BoundingBox;
+	GLEPoint m_Size, m_BoundingBox, m_BoundingBoxOrigin;
 	GLERCVector<GLEDrawObject> m_NewObjs;
+	string m_PostScript;
 	int m_CurrObject;
 public:
 	GLEScript();
@@ -1079,9 +1139,12 @@ public:
 
 	inline const GLEPoint& getSize() { return m_Size; }
 	inline const GLEPoint& getBoundingBox() { return m_BoundingBox; }
+	inline const GLEPoint& getBoundingBoxOrigin() { return m_BoundingBoxOrigin; }
+
 
 	inline void setSize(double wd, double hi) { m_Size.setXY(wd, hi); }
 	inline void setBoundingBox(double wd, double hi) { m_BoundingBox.setXY(wd, hi); }
+	inline void setBoundingBoxOrigin(double x, double y) { m_BoundingBoxOrigin.setXY(x, y); }
 
 	inline void resetObjectIterator() { m_CurrObject = 0; }
 	GLEDrawObject* nextObject();
@@ -1099,10 +1162,12 @@ public:
 	inline int getNumberNewObjects() { return m_NewObjs.size(); }
 	inline GLEDrawObject* getNewObject(int i) { return m_NewObjs.get(i); }
 	inline void addNewObject(GLEDrawObject* obj) { m_NewObjs.add(obj); }
-	void DLLFCT clearNewObjects();
+	void clearNewObjects();
+	virtual const char* getPostScriptCode();
+	virtual string* getRecordedBytesBuffer(int device);
 };
 
-class GLEColorList {
+class DLLCLASS GLEColorList {
 protected:
 	GLERCVector<GLEColor> m_Colors;
 	StringIntHash m_ColorHash;
@@ -1118,6 +1183,7 @@ public:
 	void defineOldColor(const char* name, unsigned int value);
 	void defineOldColor(const string& name, unsigned int value);
 	GLEColor* get(const string& name);
+	DLLFCT GLEColor* getSafeDefaultBlack(const string& name);
 	inline int getNbColors() { return m_Colors.size(); }
 	inline GLEColor* getColor(int i) { return m_Colors.get(i); }
 	void defineDefaultColors();
@@ -1127,7 +1193,7 @@ private:
 	void defineOldGLEColors();
 };
 
-GLEColorList* GLEGetColorList();
+GLEColorList* DLLFCT GLEGetColorList();
 
 class GLEStringSettings {
 public:
