@@ -95,6 +95,16 @@ void tex_term();
 void do_show_info();
 void get_out_name(const string& name, CmdLineObj& cmdline, GLEFileLocation* outname);
 
+#include "glearray.h"
+
+void gle_cat_csv(vector<string>* files) {
+	for (unsigned int i = 0; i < files->size(); i++) {
+		string fname(files->at(i));
+		GLECSVData csvData;
+		csvData.read(fname);
+	}
+}
+
 #if defined(HAVE_LIBGLE) && !defined(HAVE_LIBGLE_STATIC)
 int GLEMain(int argc, char **argv) {
 #else
@@ -116,6 +126,7 @@ int main(int argc, char **argv) {
 		if (g_CmdLine.hasError()) {
 			return -1;
 		}
+
 		/* Has calculator option? */
 		if (g_CmdLine.hasOption(GLE_OPT_CALC)) {
 			if (g_CmdLine.getNbMainArgs() == 0) {
@@ -126,6 +137,12 @@ int main(int argc, char **argv) {
 			return 0;
 		}
 
+		/* Cat CSV file to stdout? */
+		if (g_CmdLine.hasOption(GLE_OPT_CATCSV) && g_CmdLine.getNbMainArgs() != 0) {
+			gle_cat_csv(g_CmdLine.getMainArgs());
+			return 0;
+		}
+
 		/* Should regenerate inittex.ini? */
 		do_gen_inittex(g_CmdLine, g_Options);
 
@@ -264,15 +281,13 @@ void do_run_other_version(ConfigCollection& coll, int argc, char **argv) {
 }
 
 void do_show_info() {
-	string version;
+	string version, bdate;
 	g_get_version(&version);
+	g_get_build_date(&bdate);
 	cout << "GLE version:   " << version << endl;
-#if defined(__TIME__) && defined(__DATE__)
-	string bdate = __DATE__;
-	bdate += " "; bdate += __TIME__;
-	str_replace_all(bdate, "  ", " ");
-	cout << "Build date:    " << bdate << endl;
-#endif
+	if (!bdate.empty()) {
+		cout << "Build date:    " << bdate << endl;
+	}
 	cout << "GLE_TOP:       " << GLE_TOP_DIR << endl;
 	cout << "GLE_BIN:       " << GLE_BIN_DIR << endl;
 	/* Location of GhostScript */
@@ -391,19 +406,24 @@ void init_option_args(CmdLineObj& cmdline) {
 	setarg->addPossibleValue("jpg");
 	setarg->addPossibleValue("png");
 	setarg->addPossibleValue("x11");
+	setarg->addPossibleValue("emf");
 #ifndef HAVE_X11
 	setarg->setUnsupportedValue(GLE_DEVICE_X11);
 #endif
 #ifndef HAVE_CAIRO
 	setarg->setUnsupportedValue(GLE_DEVICE_SVG);
+	setarg->setUnsupportedValue(GLE_DEVICE_EMF);
+#endif
+#ifndef __WIN32__
+	setarg->setUnsupportedValue(GLE_DEVICE_EMF);
 #endif
 	setarg->addDefaultValue(GLE_DEVICE_EPS);
 	option->addArg(setarg);
 	cmdline.addOption(option, GLE_OPT_DEVICE);
 #ifdef HAVE_CAIRO
 	option = new CmdLineOption("cairo");
-    option->setHelp("Use cairo output device");
-    cmdline.addOption(option, GLE_OPT_CAIRO);
+	option->setHelp("Use cairo output device");
+	cmdline.addOption(option, GLE_OPT_CAIRO);
 #endif
 	option = new CmdLineOption("fullpage");
 	option->setHelp("Selects full page output");
@@ -464,6 +484,9 @@ void init_option_args(CmdLineObj& cmdline) {
 	option = new CmdLineOption("calc", "c");
 	option->setHelp("Runs GLE in \"calculator\" mode");
 	cmdline.addOption(option, GLE_OPT_CALC);
+	option = new CmdLineOption("catcsv", "csv");
+	option->setHelp("Pretty print a CSV file to standard output");
+	cmdline.addOption(option, GLE_OPT_CATCSV);
 	option = new CmdLineOption("output", "o");
 	option->setHelp("Specifies the name of the output file");
 	strarg = new CmdLineArgString("name");
@@ -471,6 +494,14 @@ void init_option_args(CmdLineObj& cmdline) {
 	strarg->setCardLimits(1, 1);
 	option->addArg(strarg);
 	cmdline.addOption(option, GLE_OPT_OUTPUT);
+	option = new CmdLineOption("nosave");
+	option->setHelp("Don't write output file to disk (dry-run)");
+	cmdline.addOption(option, GLE_OPT_NOSAVE);
+#if defined(__WIN32__) and defined(HAVE_CAIRO)
+	option = new CmdLineOption("copy");
+	option->setHelp("Copy resulting figure to clipboard (only EMF)");
+	cmdline.addOption(option, GLE_OPT_COPY);
+#endif
 	option = new CmdLineOption("compatibility", "cm");
 	option->setHelp("Selects a GLE compatibility mode");
 	strarg = new CmdLineArgString("version");
@@ -752,9 +783,10 @@ GLELoadOneFileManager::~GLELoadOneFileManager() {
 void GLELoadOneFileManager::update_bounding_box() {
 	double size_x, size_y;
 	g_get_usersize(&size_x, &size_y);
+	m_Script->setSize(size_x, size_y);
 	PSGLEDevice* psdev = (PSGLEDevice*)g_get_device_ptr();
 	m_Script->setBoundingBox(psdev->getBoundingBox()->getX(), psdev->getBoundingBox()->getY());
-	m_Script->setSize(size_x, size_y);
+	m_Script->setBoundingBoxOrigin(0.0, 0.0);
 }
 
 bool GLELoadOneFileManager::process_one_file_eps() throw(ParserError) {
@@ -795,6 +827,10 @@ bool GLELoadOneFileManager::process_one_file_eps() throw(ParserError) {
 		}
 		iter++;
 	} while (done == 1);
+	/* get recorded PostScript code */
+	m_Device->getRecordedBytes(m_Script->getRecordedBytesBuffer(GLE_DEVICE_EPS));
+	update_bounding_box();
+	/* do TeX stuff */
 	interface->checkObjectDimensions();
 	if (m_CmdLine->hasOption(GLE_OPT_CREATE_INC)) {
 		interface->createInc(m_CmdLine->getStringValue(GLE_OPT_TEXINCPREF));
@@ -845,20 +881,22 @@ void GLELoadOneFileManager::create_latex_eps_ps_pdf() throw(ParserError) {
 	FileNameDotToUnderscore(m_IncName.getFullPathNC());
 	bool create_inc = m_CmdLine->hasOption(GLE_OPT_CREATE_INC);
 	bool has_pdftex = has_pdflatex(m_CmdLine);
+	int dpi = m_CmdLine->getIntValue(GLE_OPT_DPI);
 	CmdLineArgSet* device = (CmdLineArgSet*)m_CmdLine->getOption(GLE_OPT_DEVICE)->getArg(0);
 	/* Create "_inc.eps" */
-	if (m_HasEPSFile) {
-		/* FIXME: replace with move file? */
-		m_HasIncEPS = true;
-		GLECopyFile(m_OutName->getFullPath() + ".eps", m_IncName.getFullPath() + ".eps");
+	if (device->hasOnlyValue(GLE_DEVICE_PDF) && (has_pdftex || create_inc)) {
+		/* "_inc.eps" not required in this case */
 	} else {
-		if (device->hasOnlyValue(GLE_DEVICE_PDF) && (has_pdftex || create_inc)) {
-			/* "_inc.eps" not required in this case */
-		} else {
-			m_HasIncEPS = true;
-			m_Device->writeRecordedOutputFile(m_IncName.getFullPath());
-		}
+		m_HasIncEPS = true;
+		m_Device->writeRecordedOutputFile(m_IncName.getFullPath());
 	}
+	/* Create "_inc.pdf" */
+	if (device->hasValue(GLE_DEVICE_PDF) && (has_pdftex || create_inc)) {
+		m_HasIncPDF = true;
+		create_pdf_file_ghostscript(&m_IncName, dpi, m_Script);
+		do_output_type(".pdf");
+	}
+	/* Exit if no EPS/PS/PDF required */
 	if (!requires_tex_eps(device, m_CmdLine) &&
 	    !device->hasValue(GLE_DEVICE_PS) &&
 	    !device->hasValue(GLE_DEVICE_PDF)) {
@@ -870,7 +908,7 @@ void GLELoadOneFileManager::create_latex_eps_ps_pdf() throw(ParserError) {
 	/* LaTeX does not like to run with absolute paths */
 	GLEChDir(out_name_dir);
 	if (requires_tex_eps(device, m_CmdLine)) {
-		create_eps_file_latex_dvips(out_name_np);
+		create_eps_file_latex_dvips(out_name_np, m_Script);
 		m_HasEPSFile = true;
 	}
 	if (device->hasValue(GLE_DEVICE_PS)) {
@@ -878,51 +916,33 @@ void GLELoadOneFileManager::create_latex_eps_ps_pdf() throw(ParserError) {
 		if (m_OutName->isStdout()) cat_stdout_and_del(".ps");
 		do_output_type(".ps");
 	}
-	if (device->hasValue(GLE_DEVICE_PDF)) {
-		int dpi = m_CmdLine->getIntValue(GLE_OPT_DPI);
-		if (has_pdftex || create_inc) {
-			/* Create "_inc.pdf" */
-			m_HasIncPDF = true;
-			istream* epss = m_HasIncEPS ? NULL : ((PSGLEDevice*)m_Device)->getRecordedPostScript();
-			create_pdf_file_ghostscript(&m_IncName, epss, dpi, m_Script->getBoundingBox(), false);
+	if (device->hasValue(GLE_DEVICE_PDF) && !create_inc) {
+		if (has_pdftex) {
+			create_pdf_file_pdflatex(out_name_np);
+		} else {
+			create_pdf_file_ghostscript(m_OutName, dpi, m_Script);
 			do_output_type(".pdf");
 		}
-		if (!create_inc) {
-			if (has_pdftex) {
-				create_pdf_file_pdflatex(out_name_np);
-			} else {
-				create_pdf_file_ghostscript(m_OutName, NULL, dpi, m_Script->getBoundingBox(), true);
-				do_output_type(".pdf");
-			}
-			if (m_OutName->isStdout()) cat_stdout_and_del(".pdf");
-		}
+		if (m_OutName->isStdout()) cat_stdout_and_del(".pdf");
 	}
 	GLEChDir(m_Script->getLocation()->getDirectory());
 }
 
 istream* GLELoadOneFileManager::get_eps_stream() {
-	if (m_HasEPSFile) return NULL;
-	if (m_HasTeXFile) return NULL;
-	return ((PSGLEDevice*)m_Device)->getRecordedPostScript();
+	return NULL;
 }
 
 void GLELoadOneFileManager::write_eps() throw (ParserError) {
-	if (m_CmdLine->hasOption(GLE_OPT_CREATE_INC)) {
+	if (m_CmdLine->hasOption(GLE_OPT_CREATE_INC) || m_CmdLine->hasOption(GLE_OPT_NOSAVE)) {
 		return;
 	}
-	if (m_HasEPSFile) {
-		if (m_OutName->isStdout()) {
-			/* Copy .eps from file to stdout */
-			cat_stdout(".eps");
-		}
+	if (m_OutName->isStdout()) {
+		/* Write .eps data to stdout */
+		string* buffer = m_Script->getRecordedBytesBuffer(GLE_DEVICE_EPS);
+		cout.write(buffer->data(), buffer->size());
 	} else {
-		if (m_OutName->isStdout()) {
-			/* Write .eps data to stdout */
-			m_Device->writeRecordedOutput(cout);
-		} else {
-			/* Write .eps data to output file */
-			m_Device->writeRecordedOutputFile(m_OutName->getFullPath());
-		}
+		/* Write .eps data to output file */
+		m_Device->writeRecordedOutputFile(m_OutName->getFullPath(), m_Script->getRecordedBytesBuffer(GLE_DEVICE_EPS));
 	}
 }
 
@@ -1009,15 +1029,13 @@ void load_one_file_sub(GLEScript* script, CmdLineObj& cmdline, size_t* exit_code
 			return;
 		}
 		/* Get device information */
-		istream* epss = manager.get_eps_stream();
-		manager.update_bounding_box();
 		int dpi = cmdline.getIntValue(GLE_OPT_DPI);
 		/* File has TeX code -> move it and create again */
 		if (has_tex) {
 			manager.create_latex_eps_ps_pdf();
 		} else {
 			if (device->hasValue(GLE_DEVICE_PDF)) {
-				create_pdf_file_ghostscript(&out_name, epss, dpi, script->getBoundingBox(), false);
+				create_pdf_file_ghostscript(&out_name, dpi, script);
 				manager.do_output_type(".pdf");
 			}
 		}
@@ -1026,8 +1044,7 @@ void load_one_file_sub(GLEScript* script, CmdLineObj& cmdline, size_t* exit_code
 		bool is_transp = cmdline.hasOption(GLE_OPT_TRANSPARENT);
 		for (int i = 0; i < device->getNbValues(); i++) {
 			if (is_bitmap_device(i) && device->hasValue(i)) {
-				const GLEPoint& bb = script->getBoundingBox();
-				create_bitmap_file(&out_name, epss, i, dpi, bb, is_bw, is_transp, has_tex);
+				create_bitmap_file(&out_name, i, dpi, is_bw, is_transp, script);
 				manager.do_output_type(g_device_to_ext(i));
 			}
 		}
@@ -1060,6 +1077,21 @@ void load_one_file_sub(GLEScript* script, CmdLineObj& cmdline, size_t* exit_code
 		g_select_device(GLE_DEVICE_X11);
 		DrawIt(script, &out_name, &cmdline);
 	}
+#if defined(__WIN32__) && defined(HAVE_CAIRO)
+	if (device->hasValue(GLE_DEVICE_EMF)) {
+		GLECairoDeviceEMF* dev = (GLECairoDeviceEMF*)g_select_device(GLE_DEVICE_EMF);
+		if (cmdline.hasOption(GLE_OPT_COPY)) {
+			dev->setCopyClipboard(true);
+		}
+		if (cmdline.hasOption(GLE_OPT_DPI)) {
+			int dpi = ((CmdLineArgInt*)cmdline.getOption(GLE_OPT_DPI)->getArg(0))->getValue();
+			dev->setDPI(dpi);
+		} else {
+			dev->setDPI(2000);
+		}
+		DrawIt(script, &out_name, &cmdline);
+	}
+#endif
 }
 
 void load_one_file(const char* name, CmdLineObj& cmdline, size_t* exit_code) {
