@@ -78,6 +78,7 @@ int letline[MAX_NUMBER_OF_LET_COMMANDS];
 int nlet;
 
 int g_nbar = 0;
+int g_graph_background = GLE_FILL_CLEAR;
 
 int freedataset(int i);
 void free_temp(void);
@@ -89,8 +90,9 @@ void do_axis_part_all(int xset) throw (ParserError);
 bool do_remaining_entries(int ct);
 void graph_freebars();
 double get_next_exp(TOKENS tk,int ntk,int *curtok);
+void data_command();
 
-axis_struct xx[GLE_AXIS_MAX+1];
+GLEAxis xx[GLE_AXIS_MAX+1];
 
 vector<int> g_fcalls;
 vector<int> g_funder;
@@ -110,15 +112,27 @@ bool check_axis_command_name(const char* name, const char* cmp) {
 	return false;
 }
 
+void ensureDataSetCreated(int d) {
+	if (dp[d] == NULL) {
+		dp[d] = new GLEDataSet();
+		copy_default(d);
+		if (ndata < d) ndata = d;
+	}
+}
+
+void ensureDataSetCreatedAndSetUsed(int d) {
+	ensureDataSetCreated(d);
+	dp[d]->axisscale = true;
+}
+
 // Font sizes in a graph are based in g_fontsz, but this was set in the original
 // GLE to a fixed proportion of the largest dimension of a graph. The size of an
 // axis is defined in terms of its base size, which is equal to g_fontsz.
 // A useful value for base is 0.25
 
 void begin_graph(int *pln , int *pcode , int *cp) throw (ParserError) {
-	int ct,i,ncol = 0,np,*m,t,d;
-	int d2,st;
-	double *px,*py;
+	int ct,i,t,d;
+	int st;
 	int erflg;
 	stringstream err;
 
@@ -176,6 +190,10 @@ do_next_line:
 		else kw("COLORMAP") goto do_colormap;
 		else kw("TITLE") goto do_main_title;
 		else kw("UNDER") g_funder.push_back((*pln)-1);
+		else kw("BACKGROUND")
+		{
+			g_graph_background = next_color;
+		}
 		else kw("!") goto do_next_line;
 		else kw(" ") goto do_next_line;
 		else if (check_axis_command_name(tk[ct],"NOTICKS")) goto do_noticks;
@@ -200,7 +218,7 @@ do_bar:
 		dist exp, width exp, LSTYLE 3,445,1
 		3dbar .5 .5 top black,black,black side red,green,blue  notop */
 {
-	int ng = 0,fi,di;
+	int ng = 0,fi;
 	char *ss;
 	g_nbar++;
 	br[g_nbar] = new bar_struct();
@@ -211,6 +229,7 @@ do_bar:
 	  if (toupper(*ss)=='D') {
 		ng = (br[g_nbar]->ngrp)++;
 		d = get_dataset_identifier(ss);
+		ensureDataSetCreatedAndSetUsed(d);
 		br[g_nbar]->to[ng] = d;
  	  }
 	  ss = strtok(0,",");
@@ -248,7 +267,8 @@ do_bar:
 		ss = strtok(tk[ct],",");
 		while (ss!=NULL) {
 		  if (toupper(*ss)=='D') {
-			di = get_dataset_identifier(ss);
+			int di = get_dataset_identifier(ss);
+			ensureDataSetCreatedAndSetUsed(di);
 			br[g_nbar]->from[fi++] = di;
 		  }
 		  ss = strtok(0,",");
@@ -301,7 +321,6 @@ do_fill:
 		strcpy(s2,ss);
 		strtok(0,",");
 	}
-
 	if (str_i_equals(s1,"X1")) {
 		fd[nfd]->type = 1;
 		d = get_dataset_identifier(s2);
@@ -313,7 +332,7 @@ do_fill:
 	} else if (!str_i_equals(s2,"")) {
 		fd[nfd]->type = 3;
 		d = get_dataset_identifier(s1);
-		d2 = get_dataset_identifier(s2);
+		int d2 = get_dataset_identifier(s2);
 		fd[nfd]->da = d;
 		fd[nfd]->db = d2;
 	} else if (toupper(*s1)=='D') {
@@ -323,14 +342,14 @@ do_fill:
 	} else {
 		g_throw_parser_error("invalid fill option, wanted d1,d2 or x1,d1 or d1,x2 or d1");
 	}
+	if (fd[nfd]->da != 0) ensureDataSetCreatedAndSetUsed(fd[nfd]->da);
+	if (fd[nfd]->db != 0) ensureDataSetCreatedAndSetUsed(fd[nfd]->db);
 	ct++;
-
 	fd[nfd]->color = g_get_grey(1.0-nfd*.1);
-	fd[nfd]->xmin = -LARGE_NUM;
-	fd[nfd]->xmax = LARGE_NUM;
-	fd[nfd]->ymin = -LARGE_NUM;
-	fd[nfd]->ymax = LARGE_NUM;
-
+	fd[nfd]->xmin = -GLE_INF;
+	fd[nfd]->xmax = GLE_INF;
+	fd[nfd]->ymin = -GLE_INF;
+	fd[nfd]->ymax = GLE_INF;
 	while (ct<=ntk)  {
 		kw("COLOR") 	{ct++; fd[nfd]->color = pass_color(tk[ct]);}
 	   else kw("XMIN") 	fd[nfd]->xmin = next_exp;
@@ -410,342 +429,10 @@ do_key:
 	}
 goto do_next_line;
 /*----------------------------------------------------------------*/
-/* data a.dat				*/
-/* data a.dat d2 d5			*/
-/* data a.dat d1=c1,c3  d2=c5,c1	*/
-/* data a.dat IGNORE n d2 d5 */
-/* data a.dat IGNORE n d1=c1,c3  d2=c5,c1	*/
-/* data a.dat COMMENT # d1=c1,c3  d2=c5,c1	*/
-/* data a.dat COMMENT # IGNORE n d1=c1,c3  d2=c5,c1	*/
-
 do_data:
 {
-
-// FIXME:
-// Rewrite using new Tokenizer
-// Avoid duplicate code for "extract=true/false for COMMENT, NOX, ..." !!!
-
-	static int data_d[MAX_NUMBER_OF_ITEMS],data_x[MAX_NUMBER_OF_ITEMS],data_y[MAX_NUMBER_OF_ITEMS];
-	static bool xy_given[MAX_NUMBER_OF_ITEMS];
-	int nm=0,expect_col=0,extract=0,xi,yi,dn,j,nncol=0,allind,ignore=0;
-	char *s1,buff1[30];
-	bool nox = false;
-	string comment = "";
-	string infile;
-	allind = 0;
-	xy_given[0] = false;
-	for (i = 1; i < MAX_NUMBER_OF_ITEMS; i++) { /* default mapping if no dataset names given */
-		data_d[i]   = freedataset(i);
-		              // if multiple "data" commands in file, then number sequentially
-		              // this is what freedataset does
-		data_x[i]   = 1;
-		data_y[i]   = i+1;
-		              // column in data file for x and y values
-		xy_given[i] = false;
-		              // true if dn=c1,c2 given for this dataset
-	}
-	while (ct<=ntk)  { if (str_i_str(tk[ct],"=")!=NULL) extract = true; ct++; }
-	/* goto third token */
-	ct = 3;
-	if (extract == static_cast<int>(true)) {
-	 while (ct<=ntk)  {
-		if (toupper(*tk[ct])=='D') {
-			nm++;
-			strncpy(buff1,tk[ct],29);
-			s1 = strtok(buff1,"=");
-			if (s1!=NULL) data_d[nm] = get_dataset_identifier(s1);
-			if (s1==NULL || data_d[nm]==0) {
-				g_throw_parser_error("expecting Dn=Ca,Cb (no spaces), not '", tk[ct], "'");
-			}
-			s1 = strtok(NULL,",");
-			if (s1!=NULL) {
-				xy_given[nm] = true;
-				data_x[nm] = atoi(s1+1); // a
-			}
-			s1 = strtok(NULL,",");
-			if (s1 != NULL && *s1=='*') {
-				allind = data_d[nm];
-				// FIXME: what does "allind" mean?
-				// Also not in manual and unclear -> drop this feature!
-			} else {
-			 if (s1!=NULL) data_y[nm] = atoi(s1+1); //b
-			 if (s1==NULL || data_x[nm]==0 || data_y[nm]==0) {
-				g_throw_parser_error("expecting dn=ci,cj (no spaces), not '", tk[ct], "'");
-			 }
-			 if (data_x[nm]>expect_col) expect_col = data_x[nm];
-			 if (data_y[nm]>expect_col) expect_col = data_y[nm];
-		 	}
-		}
-		else if(str_i_equals(tk[ct],"IGNORE"))
-		{	/* user wants to ignore some lines */
-			ignore = -1;
-			ignore = atoi(tk[++ct]);
-			if(ignore < 0) g_throw_parser_error("expecting ignore n, where n is an integer, not '", buff1, "'");
-		}
-		else if(str_i_equals(tk[ct],"COMMENT"))
-		{	/*  user wants to ignore lines that begin with a comment */
-			comment = "";
-			comment = tk[++ct];
-			if(comment.length() == 0) g_throw_parser_error("expecting comment character, not '", buff1, "'");
-		}
-		else if(str_i_equals(tk[ct],"NOX"))
-		{
-			nox = true;
-		}
-		else
-		{
-			if (*tk[ct] != 0) g_throw_parser_error("expecting dn=ci,cj (no spaces), not '", tk[ct], "'");
-		}
-	   	ct++;
-	 }
-	} else {
-	 while (ct<=ntk)  {
-		if (*tk[ct]==' ') ct++;
-		if (toupper(*tk[ct])=='D') {
-			nm++;
-			data_d[nm] = get_dataset_identifier(tk[ct]);
-			data_x[nm] = 1;
-			data_y[nm] = nm+1;
-		}
-		else if(str_i_equals(tk[ct],"IGNORE"))
-		{	/* user wants to ignore some lines */
-			ignore = -1;
-			ignore = atoi(tk[++ct]);
-			if(ignore < 0) gprint("Expecting IGNORE n   where n is an integer got->{%s}\n",buff1);
-		}
-		else if(str_i_equals(tk[ct],"COMMENT"))
-		{	/* user wants to ignore lines that begin with a comment */
-			comment = "";
-			comment = tk[++ct];
-			if(comment.length() == 0) g_throw_parser_error("expecting ignore n, where n is an integer, not '", buff1, "'");
-		}
-		else if(str_i_equals(tk[ct],"NOX"))
-		{
-			nox = true;
-		}
-		else
-		{
-			gprint("Warning, found {%s}, expecting data a.dat d1 d2 d3 \n",tk[ct]);
-		}
-	   	ct++;
-	  }
-	}
-	if (expect_col==0 && nm>0) expect_col = nm+1;
-	if (allind>0) expect_col = 0;
-	ct = 2;
-	pass_file_name(tk[2], infile);
-	validate_file_name(infile, true);
-	ifstream fptr;
-	fptr.open(infile.c_str());
-	if (!fptr.is_open()) {
-		g_throw_parser_error_sys("unable to open data file '", infile.c_str(), "'");
-	}
-
-	int line = 0;
-	vector<string> temp_s;
-	vector<bool> temp_miss;
-	vector<double> temp_data;
-
-	TokenizerLanguage tok_lang;
-	StringTokenizer str_tok(&tok_lang);
-	tok_lang.setSpaceTokens(" ,;\t");
-	tok_lang.setLineCommentTokens("!");
-	tok_lang.setDecimalDot('.');
-	tok_lang.setParseStrings(true);
-	str_tok.set_fname(infile.c_str());
-
-	/* loop through each line in the file*/
-	while (fptr.good()) {
-
-		string large_inbuff;
-		getline(fptr, large_inbuff);
-		str_trim_right(large_inbuff);
-
-		/* ignore the lines specified by user */
-		if (line >= ignore) {
-			str_tok.set_string(large_inbuff);
-			str_tok.inc_line();
-
-			ncol = 0;
-			/* count the number of columns and put in tmpf and tmpmiss */
-			while (true) {
-				string& token = str_tok.try_next_token();
-				// must compare first porton of next token
-				if (token == "" || (comment.length() > 0 &&  token.find(comment) == 0 ) ) {
-					break;
-				}
-				ncol++;
-				if (token == "*" || token == "?" || token == "-" || token == ".") {
-					temp_miss.push_back(true);
-					temp_data.push_back(0);
-					temp_s.push_back("*");
-				} else {
-					temp_miss.push_back(false);
-					temp_data.push_back(atof(token.c_str()));
-					temp_s.push_back(token);
-				}
-			}
-
-			if (nncol == 0) nncol = ncol;
-			if (ncol != nncol && ncol > 0) {
-				if (allind==0) {
-					stringstream err;
-					err << "inconsistent number of columns " << ncol << " <> " << nncol;
-					err << " at line " << (line+1);
-					err << " in '" << infile << "'";
-					g_throw_parser_error(err.str());
-				}
-			}
-			if (nm==0 && ncol>1) nm = ncol-1;
-
-		} /* end if ignore */
-		line++;
-	}
-	fptr.close();
-	/* file is closed now resort data in tmpf and tmpfmiss*/
-	if (expect_col > 0 && expect_col > nncol)
-	{
-		ostringstream err;
-		err << "expecting " << expect_col << " columns, but found " << nncol << " in data file '" << infile << "'";
-		g_throw_parser_error(err.str());
-	}
-
-	np = nncol > 0 ? temp_data.size() / nncol : 0;
-	if (allind>0)
-	{
-		/* if data file contains y values only, y1, y2 ,y3...*/
-		dn = allind;
-		np = temp_data.size();
-		if (dn>ndata) ndata = dn;
-		if (dp[dn]==NULL)
-		{
-			dp[dn] = new GLEDataSet();
-			copy_default(dn);
-		}
-		np++;
-		dp[dn]->clearAll();
-		dp[dn]->xv = (double*) myallocz(sizeof(*px)*np);
-		dp[dn]->yv = (double*) myallocz(sizeof(*px)*np);
-		dp[dn]->miss = (int*) myallocz(sizeof(*m)*np);
-		dp[dn]->yv_str = NULL;
-		np--;
-		px = dp[dn]->xv;
-		py = dp[dn]->yv;
-		m = dp[dn]->miss;
-
-	 	for (i=1;i<=np;i++)
-		{
-			*(px++) = i;
-			*(py++) = temp_data[i-1];
-			*(m++) = temp_miss[i-1];
-		}
-		dp[dn]->np = np;
-		goto do_next_line;
-	}
-
-	// Handle special types of data sets
-	if (nncol == 1) {
-		nox = true;
-	}
-	if (nox) {
-		nm++;
-		for (i = 1; i < MAX_NUMBER_OF_ITEMS; i++) {
-			if (!xy_given[i]) {
-				data_y[i]   = i;
-			}
-		}
-	}
-	bool all_str_col = true;
-	if (nox) {
-		// no x-values implies also no labels from first data column
-		all_str_col = false;
-	} else {
-		for (int row = 0; row < np; row++) {
-			if (is_float_miss(temp_s[row*nncol])) {
-				all_str_col = false;
-				break;
-			}
-		}
-	}
-	int first_ds = 1;
-	if (all_str_col) {
-		nox = true;
-		first_ds = 0;
-		data_y[0] = 1;
-		data_d[0] = freedataset(0);
-		xx[GLE_AXIS_X].setNamesDataSet(0);
-	}
-	bool has_header = true;
-	for (int col = 0; col < nncol; col++) {
-		if (is_float_miss(temp_s[col])) {
-			has_header = false;
-			break;
-		}
-	}
-	int first_row = 1;
-	if (has_header) {
-		first_row = 2;
-		for (j=1;j<=nm;j++) {
-			dn = data_d[j];
-			yi = data_y[j];
-			if (dn>ndata) ndata = dn;
-			if (dp[dn]==NULL) {
-				dp[dn] = new GLEDataSet();
-				copy_default(dn);
-			}
-			str_remove_quote(temp_s[yi-1]);
-			dp[dn]->key_name = sdup(temp_s[yi-1].c_str());
-		}
-	}
-
-	for (j=first_ds;j<=nm;j++)
-	{
-		dn = data_d[j];
-		xi = data_x[j];
-		yi = data_y[j];
-		if (dn>ndata) ndata = dn;
-		if (dp[dn]==NULL) {
-			dp[dn] = new GLEDataSet();
-			copy_default(dn);
-		}
-		np++;
-		dp[dn]->clearAll();
-		dp[dn]->xv = (double*) myallocz(sizeof(*px)*np);
-		dp[dn]->yv = (double*) myallocz(sizeof(*px)*np);
-		dp[dn]->miss = (int*) myallocz(sizeof(*m)*np);
-		dp[dn]->yv_str = new vector<string>();
-		np--;
-		px = dp[dn]->xv;
-		py = dp[dn]->yv;
-		m = dp[dn]->miss;
-
-		// cout << "nncol = " << nncol << " xi = " << xi << " yi = " << yi << endl;
-
-		if (nox && !xy_given[j]) {
-			for (i = first_row; i <= np ; i++) {
-				int y_offs = i*nncol-nncol+yi-1;
-				*(px++) = i-first_row+1;
-				*(py++) = temp_data[y_offs];
-				// yv_str is used to use this data set as labels
-				str_remove_quote(temp_s[y_offs]);
-				dp[dn]->yv_str->push_back(temp_s[y_offs]);
-				*(m++) = (temp_miss[y_offs] == true);
-			}
-		} else {
-			for (i = first_row; i <= np ; i++) {
-				int y_offs = i*nncol-nncol+yi-1;
-				int x_offs = i*nncol-nncol+xi-1;
-				*(px++) = temp_data[x_offs];
-				*(py++) = temp_data[y_offs];
-				// yv_str is used to use this data set as labels
-				str_remove_quote(temp_s[y_offs]);
-				dp[dn]->yv_str->push_back(temp_s[y_offs]);
-				*(m++) = (temp_miss[x_offs] == true || temp_miss[y_offs] == true);
-			}
-		}
-		dp[dn]->np = np-first_row+1;
-	}
-
+   data_command();
+   goto do_next_line;
 }
 goto do_next_line;
 /*----------------------------------------------------------------*/
@@ -893,12 +580,7 @@ do_title:
 do_datasets:
 	d = get_dataset_identifier(tk[1]); /* dataset number (right$(k$,2))  (0=dn) */
 	if (d != 0) {
-		if (dp[d] == NULL) {
-			dp[d] = new GLEDataSet();
-			copy_default(d);
-			if (ndata < d) ndata = d;
-		}
-		dp[d]->axisscale = true;
+		ensureDataSetCreatedAndSetUsed(d);
 		do_dataset(d);
 	} else {
 		for (d = 0; d < MAX_NB_DATA; d++) {
@@ -969,9 +651,6 @@ void do_axis(int axis, bool craxis) throw (ParserError) {
 		}
 		else kw("NEGATE") {
 			xx[axis].negate = true;
-			if (axis == 1 || axis == 2) {
-				data_negate[axis] = true;
-			}
 		}
 		else kw("FONT") xx[axis].label_font = next_font;
 		else kw("LOG") xx[axis].log = true;  /* don't make log depend on craxis, otherwise grid may not line up */
@@ -1293,10 +972,10 @@ void set_sizelength() {
 	graph_y1 = ybl;
 	graph_x2 = xbl + xlength;
 	graph_y2 = ybl + ylength;
-	graph_xmin = wxmin;
-	graph_xmax = wxmax;
-	graph_ymin = wymin;
-	graph_ymax = wymax;
+	graph_xmin = xx[GLE_AXIS_X].getMin();
+	graph_xmax = xx[GLE_AXIS_X].getMax();
+	graph_ymin = xx[GLE_AXIS_Y].getMin();
+	graph_ymax = xx[GLE_AXIS_Y].getMax();
 }
 
 void axis_init_length() {
@@ -1314,10 +993,11 @@ void axis_add_grid() {
 			double len = axis_horizontal(i) ? ylength : xlength;
 			if (!xx[i].hasGridOnTop()) xx[i].setGrid(true);
 			xx[i].ticks_length = len;
-			xx[i].subticks_length = len;
-			/* no ticks on second axis */
 			xx[i+2].ticks_off = true;
-			xx[i+2].subticks_off = true;
+			if (xx[i].subticks_length == 0.0) {
+				xx[i].subticks_length = len;
+				xx[i+2].subticks_off = true;
+			}
 			if (!xx[i].has_subticks_onoff) {
 				/* only change default behaviour if no explicit setting given */
 				if (xx[i].log) {
@@ -1540,6 +1220,15 @@ void draw_graph(KeyInfo* keyinfo) throw (ParserError) {
 	/* Throw away missing values if NOMISS on datasets */
 	gr_thrownomiss();
 
+	/* Draw graph background */
+	if (g_graph_background != (int)GLE_FILL_CLEAR) {
+		int old_fill;
+		g_get_fill(&old_fill);
+		g_set_fill(g_graph_background);
+		g_box_fill(graph_x1, graph_y1, graph_x2, graph_y2);
+		g_set_fill(old_fill);
+	}
+
 	if (g_colormap != NULL) {
 		g_colormap->setXRange(xx[1].getMin(), xx[1].getMax());
 		g_colormap->setYRange(xx[2].getMin(), xx[2].getMax());
@@ -1628,8 +1317,8 @@ void graph_freebars() {
 }
 
 void graph_init() {
+	g_graph_background = GLE_FILL_CLEAR;
 	ndata = 0; nfd = 0; g_nbar = 0;
-	data_negate[1] = data_negate[2] = 0;	/* a.r. */
 	xx[GLE_AXIS_X0].off = true;
 	xx[GLE_AXIS_Y0].off = true;
 	xx[GLE_AXIS_T].off = true;
@@ -1687,3 +1376,347 @@ int get_dataset_identifier(const char* ds, bool def) throw(ParserError) {
 	}
 	return result;
 }
+
+int get_dataset_identifier(const string& ds, GLEParser* parser, bool def) throw(ParserError) {
+	Tokenizer* tokens = parser->getTokens();
+	if (str_i_equals(ds, "d")) {
+		tokens->ensure_next_token("[");
+		int result = (int)floor(parser->evalTokenToDouble() + 0.5);
+		if (result < 0 || result >= MAX_NB_DATA) {
+			ostringstream err;
+			err << "data set identifier out of range: '" << result << "'";
+			throw tokens->error(err.str());
+		}
+		tokens->ensure_next_token("]");
+		if (def && dp[result] == NULL) {
+			ostringstream err;
+			err << "data set d" << result << " not defined";
+			throw tokens->error(err.str());
+		}
+		return result;
+	} else if (str_i_equals(ds, "dn")) {
+		return 0;
+	} else {
+		if (ds.size() <= 1 || toupper(ds[0]) != 'D') {
+			throw tokens->error("illegal data set identifier");
+		}
+		char* ptr = NULL;
+		int result = strtol(ds.c_str() + 1, &ptr, 10);
+		if (*ptr != 0) {
+			throw tokens->error("data set identifier should be integer");
+		}
+		if (result < 0 || result >= MAX_NB_DATA) {
+			throw tokens->error("data set identifier out of range");
+		}
+		if (def && dp[result] == NULL) {
+			throw tokens->error("data set '", ds.c_str(), "' not defined");
+		}
+		return result;
+	}
+}
+
+int get_column_number(GLEParser* parser) throw(ParserError) {
+	Tokenizer* tokens = parser->getTokens();
+	const string& token = tokens->next_token();
+	if (str_i_equals(token, "c")) {
+		tokens->ensure_next_token("[");
+		int result = (int)floor(parser->evalTokenToDouble() + 0.5);
+		if (result < 0) {
+			ostringstream err;
+			err << "column index out of range: '" << result << "'";
+			throw tokens->error(err.str());
+		}
+		tokens->ensure_next_token("]");
+		return result;
+	} else {
+		if (token.size() <= 1 || toupper(token[0]) != 'C') {
+			throw tokens->error("illegal column index '", token.c_str(), "'");
+		}
+		char* ptr = NULL;
+		int result = strtol(token.c_str()+1, &ptr, 10);
+		if (*ptr != 0) {
+			throw tokens->error("column index should be integer, not '", token.c_str(), "'");
+		}
+		if (result < 0) {
+			throw tokens->error("column index out of range '", token.c_str(), "'");
+		}
+		return result;
+	}
+}
+
+class GLEDataSetDescription {
+public:
+	GLEDataSetDescription();
+	int ds;
+	bool xygiven;
+	int cx;
+	int cy;
+};
+
+class GLEDataDescription {
+public:
+	GLEDataDescription();
+	inline int getNbDataSets() { return m_description.size(); }
+	inline GLEDataSetDescription* getDataSet(int i) { return &m_description[i]; }
+	inline void addDataSet(const GLEDataSetDescription& description) { m_description.push_back(description); }
+
+private:
+		vector<GLEDataSetDescription> m_description;
+public:
+	string fileName;
+	string comment;
+	int ignore;
+	bool nox;
+};
+
+GLEDataSetDescription::GLEDataSetDescription() {
+	ds = 0;
+	xygiven = false;
+	cx = -1;
+	cy = -1;
+}
+
+GLEDataDescription::GLEDataDescription() {
+	ignore = -1;
+	nox = false;
+}
+
+/* data a.dat												  */
+/* data a.dat d2 d5											*/
+/* data a.dat d1=c1,c3 d2=c5,c1						  */
+/* data a.dat IGNORE n d2 d5							  */
+/* data a.dat IGNORE n d1=c1,c3 d2=c5,c1			  */
+/* data a.dat COMMENT # d1=c1,c3 d2=c5,c1				*/
+/* data a.dat COMMENT # IGNORE n d1=c1,c3 d2=c5,c1	*/
+
+void read_data_description(GLEDataDescription* description) {
+	// Get data command
+	string datacmd;
+	get_block_line(g_get_error_line(), datacmd);
+	// Initialize parser for this line
+	GLEParser* parser = get_global_parser();
+	parser->setString(datacmd.c_str());
+	Tokenizer* tokens = parser->getTokens();
+	tokens->ensure_next_token_i("DATA");
+	// Read file name
+	parser->evalTokenToFileName(&description->fileName);
+	// Parse rest of the line
+	while (true) {
+		// read next token from the data description
+		const string& token = tokens->try_next_token();
+		if (token == "") {
+			// no further tokens
+			break;
+		}
+		// check if it is one of the options
+		if (str_i_equals(token, "IGNORE")) {
+			description->ignore = tokens->next_integer();
+		} else if (str_i_equals(token, "COMMENT")) {
+			parser->evalTokenToFileName(&description->comment);
+		} else if(str_i_equals(token,"NOX")) {
+			description->nox = true;
+		} else {
+			// check if it is a data set identifier
+			GLEDataSetDescription datasetDescription;
+			datasetDescription.ds = get_dataset_identifier(token, parser, false);
+			if (tokens->is_next_token("=")) {
+				datasetDescription.xygiven = true;
+				datasetDescription.cx = get_column_number(parser);
+				tokens->ensure_next_token(",");
+				datasetDescription.cy = get_column_number(parser);
+			}
+			description->addDataSet(datasetDescription);
+		}
+	}
+}
+
+double get_data_value(const vector<string>& temp, int dn, int point, const char* xy, int offs, bool* missing) {
+	const string& value(temp[offs]);
+	if (value == "*") {
+		*missing = true;
+		return 0.0;
+	} else {
+		char *ptr;
+		double result = strtod(value.c_str(), &ptr);
+		if (*ptr != 0) {
+/*			ostringstream err;
+			err << "expected floating point number, not '" << value << "'";
+			err << ", while reading the " << xy << "-value of point " << (point + 1) << " of d" << dn;
+			g_throw_parser_error(err.str());  */
+		}
+		return result;
+	}
+}
+
+void createDataSet(int dn) {
+	if (dn > ndata) ndata = dn;
+	if (dp[dn] == NULL) {
+		dp[dn] = new GLEDataSet();
+		copy_default(dn);
+	}
+}
+
+void data_command() {
+	// Read data set description
+	GLEDataDescription description;
+	read_data_description(&description);
+	// Open file
+	ifstream fptr;
+	validate_open_input_stream(fptr, description.fileName);
+	// File data
+	int fileLine = 0;
+	int dataLine = 0;
+	int dataColumns = -1;
+	vector<string> temp;
+	// Tokenizer
+	TokenizerLanguage tok_lang;
+	StringTokenizer str_tok(&tok_lang);
+	tok_lang.setSpaceTokens(" ,;\t");
+	tok_lang.setLineCommentTokens("!");
+	tok_lang.setDecimalDot('.');
+	tok_lang.setParseStrings(true);
+	str_tok.set_fname(description.fileName.c_str());
+	// Read file line by line
+	while (fptr.good()) {
+		string line;
+		getline(fptr, line);
+		str_trim_right(line);
+		if (fileLine >= description.ignore) {
+			str_tok.set_string(line);
+			str_tok.inc_line();
+			int lineColumns = 0;
+			while (true) {
+				string& token = str_tok.try_next_token();
+				// must compare first porton of next token
+				const string& comment = description.comment;
+				if (token == "" || (comment.length() > 0 &&  token.find(comment) == 0 ) ) {
+					break;
+				}
+				if (token == "*" || token == "?" || token == "-" || token == ".") {
+					token = "*";
+				}
+				lineColumns++;
+				temp.push_back(token);
+			}
+			if (lineColumns > 0) {
+				dataLine++;
+				if (dataColumns == -1) {
+					dataColumns = lineColumns;
+				} else {
+					if (lineColumns != dataColumns) {
+						ostringstream err;
+						err << "inconsistent number of columns " << lineColumns << " <> " << dataColumns;
+						err << " at line " << (fileLine + 1);
+						err	<< " in '" << description.fileName << "'";
+						g_throw_parser_error(err.str());
+					}
+				}
+			}
+		}
+		fileLine++;
+	}
+	fptr.close();
+	// Handle special types of data sets
+	bool all_str_col = true;
+	for (int row = 0; row < dataLine; row++) {
+		if (is_float_miss(temp[row * dataColumns])) {
+			all_str_col = false;
+			break;
+		}
+	}
+	bool nox = description.nox || dataColumns == 1 || all_str_col;
+	// Auto-assign columns
+	int cx = nox ? 0 : 1;
+	int cyOffs = nox && !all_str_col ? 0 : 1;
+	int nbDataSetsGiven = description.getNbDataSets();
+	for (int i = 0; i < nbDataSetsGiven; ++i) {
+		GLEDataSetDescription* dataset = description.getDataSet(i);
+		if (!dataset->xygiven) {
+			dataset->cx = cx;
+			dataset->cy = i + cyOffs + 1;
+		}
+	}
+	if (nbDataSetsGiven == 0) {
+		int nbDataSetsMax = dataColumns - cyOffs;
+		for (int i = 0; i < nbDataSetsMax; ++i) {
+			GLEDataSetDescription dataset;
+			dataset.ds = freedataset(i + 1);
+			dataset.cx = cx;
+			dataset.cy = i + cyOffs + 1;
+			description.addDataSet(dataset);
+		}
+	}
+	// Construct names data set
+	if (all_str_col) {
+		int ds = 0;
+		GLEDataSetDescription dataset;
+		dataset.ds = ds;
+		dataset.cx = 0;
+		dataset.cy = 1;
+		description.addDataSet(dataset);
+		xx[GLE_AXIS_X].setNamesDataSet(ds);
+	}
+	// Validate column indices
+	for (int i = 0; i < description.getNbDataSets(); i++) {
+		GLEDataSetDescription* dataset = description.getDataSet(i);
+		if (dataset->cx < 0 || dataset->cx > dataColumns) {
+			ostringstream err;
+			err << "x-value column index out of range for d" << dataset->ds << ": " << dataset->cx << " not in [0,...," << dataColumns << "]";
+			g_throw_parser_error(err.str());
+		}
+		if (dataset->cy < 0 || dataset->cy > dataColumns) {
+			ostringstream err;
+			err << "y-value column index out of range for d" << dataset->ds << ": " << dataset->cy << " not in [0,...," << dataColumns << "]";
+			g_throw_parser_error(err.str());
+		}
+	}
+	// Auto-detect header
+	bool has_header = true;
+	for (int col = 0; col < dataColumns; col++) {
+		if (is_float_miss(temp[col])) {
+			has_header = false;
+			break;
+		}
+	}
+	// Read header and copy to key
+	int first_row = 0;
+	if (has_header) {
+		first_row = 1;
+		for (int i = 0; i < description.getNbDataSets(); i++) {
+			GLEDataSetDescription* dataset = description.getDataSet(i);
+			int dn = dataset->ds;
+			int cy = dataset->cy;
+			createDataSet(dn);
+			str_remove_quote(temp[cy - 1]);
+			dp[dn]->key_name = sdup(temp[cy - 1].c_str());
+		}
+	}
+	// Copy all data to the data sets
+	for (int i = 0; i < description.getNbDataSets(); i++) {
+		GLEDataSetDescription* dataset = description.getDataSet(i);
+		int dn = dataset->ds;
+		createDataSet(dn);
+		int np = dataLine - first_row;
+		dp[dn]->clearAll();
+		dp[dn]->xv = (double*) myallocz(sizeof(double) * (np + 1));
+		dp[dn]->yv = (double*) myallocz(sizeof(double) * (np + 1));
+		dp[dn]->miss = (int*) myallocz(sizeof(int) * (np + 1));
+		dp[dn]->yv_str = new vector<string>();
+		dp[dn]->np = np;
+		for (int j = 0; j < np ; j++) {
+			bool missing = false;
+			if (dataset->cx == 0) {
+				dp[dn]->xv[j] = j + 1;
+			} else {
+				int x_offs = (j + first_row) * dataColumns + dataset->cx - 1;
+				dp[dn]->xv[j] = get_data_value(temp, dn, j, "x", x_offs, &missing);
+			}
+			int y_offs = (j + first_row) * dataColumns + dataset->cy - 1;
+			dp[dn]->yv[j] = get_data_value(temp, dn, j, "y", y_offs, &missing);
+			dp[dn]->miss[j] = missing;
+			// yv_str is used to use this data set as labels
+			str_remove_quote(temp[y_offs]);
+			dp[dn]->yv_str->push_back(temp[y_offs]);
+		}
+	}
+}
