@@ -48,7 +48,6 @@
 /* for key command and gx(), gy() */
 
 #define dbg if ((gle_debug & 64)>0)
-#define LARGE_NUM 1E30
 extern int gle_debug;
 
 class KeyInfo;
@@ -61,7 +60,6 @@ void gdraw_key(KeyInfo* info);
 void copy_default(int d);
 void do_dataset(int d) throw(ParserError);
 void do_each_dataset_settings();
-void draw_errbar(double x, double y, double eup, double ewid);
 void fill_vec(double x1, double y1, double x2, double y2, vector<double>* vec);
 void do_smooth(void);
 void window_set(bool showError) throw(ParserError);
@@ -79,12 +77,9 @@ void bar_reset();
 void doskip(char *s,int *ct);
 void store_window_bounds_to_vars();
 void draw_markers(void) throw (ParserError);
-void draw_mark(double x1, double y1, int i, double sz, double dval) throw (ParserError);
 void do_dataset_key(int d);
 void do_bigfile_compatibility() throw(ParserError);
 
-/* #define fnx(vv)  (((vv-wxmin)/(wxmax-wxmin)) * xlength + xbl) */
-/* #define fny(vv)  (((vv-wymin)/(wymax-wymin)) * ylength + ybl) */
 #define kw(ss) if (str_i_equals(tk[ct],ss))
 #define true (!false)
 #define false 0
@@ -116,7 +111,11 @@ struct fill_data {
 
 class GLEDataSet;
 
+void draw_vec(double x1, double y1, double x2, double y2, GLEDataSet* ds);
+void draw_mark(double x1, double y1, int i, double sz, double dval, GLEDataSet* ds) throw (ParserError);
+
 /* range of dataset dimension is initialized in window_set */
+/* can be different from axis range because "xmin" / "xmax" / "ymin" / "ymax" settings of dn */
 
 class GLEDataSetDimension {
 protected:
@@ -164,6 +163,8 @@ public:
 	inline int* getM() { return &m_M[0]; }
 };
 
+class GLEAxis;
+
 class GLEDataSet {
 public:
 	double *xv;	/* x data values */
@@ -216,6 +217,11 @@ public:
 	void clearAll();
 	GLEDataSetDimension* getDimXInv();
 	GLEDataSetDimension* getDimYInv();
+	GLEAxis* getAxis(int i);
+	void clip(double *x, double *y);
+	bool contains(double x, double y);
+	void checkRanges() throw(ParserError);
+	void copyRangeIfRequired(int dimension);
 	inline GLEDataSetDimension* getDim(int i) { return &dims[i]; }
 };
 
@@ -260,7 +266,6 @@ GRAPHDEF double graph_x1,graph_y1,graph_x2,graph_y2;  /* in cm */
 GRAPHDEF double graph_xmin,graph_ymin,graph_xmax,graph_ymax; /* graph units */
 GRAPHDEF char ebuff[400];
 GRAPHDEF int etype,eplen;
-GRAPHDEF double wxmin,wymin,wxmax,wymax; /* in axis units the bounds of the graph */
 GRAPHDEF double xbl,ybl;
 GRAPHDEF double xlength,ylength;
 GRAPHDEF double g_xsize,g_ysize,g_hscale,g_vscale,g_fontsz;
@@ -270,20 +275,17 @@ GRAPHDEF bool g_auto_s_h, g_auto_s_v;
 GRAPHDEF bool g_math;
 GRAPHDEF double sizex,sizey;
 GRAPHDEF double vscale,hscale;
-/*GRAPHDEF int g_ynegate; */
-GRAPHDEF int data_negate[3];		/* a.r. */
 
 GRAPHDEF struct fill_data *fd[MAX_NB_FILL];
 GRAPHDEF int nfd;
 GRAPHDEF int gntmp;
 
 GRAPHDEF GLEDataSet *dp[MAX_NB_DATA];
-GRAPHDEF GLEDataSet *dpp;
 
 GRAPHDEF bar_struct *br[20];
 void vinit_axis(int i);
 void vinit_title_axis();
-void draw_bar(double x, double yf, double yt, double wd, bar_struct* barset, int di, int b) throw(ParserError);
+void draw_bar(double x, double yf, double yt, double wd, bar_struct* barset, int di, GLEDataSet* toDataSet) throw(ParserError);
 void draw_user_function_calls(bool underneath) throw(ParserError);
 void get_dataset_ranges();
 void set_bar_axis_places();
@@ -299,7 +301,7 @@ void draw_key(int nkd, struct offset_struct* koffset, char *kpos,double khei, in
 
 #define DP_CAST (struct data_struct*)
 #define BR_CAST (struct bar_struct*)
-#define AX_CAST (axis_struct*)
+#define AX_CAST (GLEAxis*)
 #define FD_CAST (struct fill_data*)
 
 class GLEZData;
