@@ -39,6 +39,7 @@
 #include "all.h"
 #include "tokens/stokenizer.h"
 #include "cutils.h"
+#include "core.h"
 #include "gprint.h"
 #include "numberformat.h"
 
@@ -367,10 +368,14 @@ void GLENumberFormatterSci::formatExpPart(int exp, string* output) {
 			*output += exp_str;
 			break;
 		case GLE_NF_SCI_10:
-			if (output->length() == 0) *output = "10^{";
-			else *output += "\\cdot 10^{";
-			*output += exp_str;
-			*output += "}";
+			{
+				ostringstream toAdd;
+				if (g_get_tex_labels()) toAdd << "$";
+				if (output->length() != 0) toAdd << "\\cdot ";
+				toAdd << "10^{" << exp_str << "}";
+				if (g_get_tex_labels()) toAdd << "$";
+				*output = toAdd.str();
+			}
 			break;
 	}
 }
@@ -478,7 +483,8 @@ void GLENumberFormatterEng::format(double number, string* output) {
 			*output = result;
 			formatExpPart(expof10, output);
 		} else {
-			sprintf(res, "%s", prefix[(expof10-GLE_FMT_ENG_PREFIX_START)/3]);
+			const char* fmt = g_get_tex_labels() ? "$\\mathrm{%s}$" : "%s";
+			sprintf(res, fmt, prefix[(expof10-GLE_FMT_ENG_PREFIX_START)/3]);
 			*output = result;
 		}
 		myDoAll(output);
@@ -505,7 +511,9 @@ void GLENumberFormatterEng::format(double number, string* output) {
 		*output = result;
 		doNoZeroes(output);
 		*output += " ";
+		if (g_get_tex_labels()) *output += "$\\mathrm{";
 		*output += prefix[(expof10-GLE_FMT_ENG_PREFIX_START)/3];
+		if (g_get_tex_labels()) *output += "}$";
 	}
 	myDoAll(output);
 }
@@ -626,7 +634,9 @@ void GLENumberFormatterFrac::format(double number, string* output) {
 				gle_int_to_string((int)floor(num+1e-7), &str);
 				*output += str;
 			}
-			if (number != 0.0) *output += "\\pi";
+			if (number != 0.0) {
+				*output += g_get_tex_labels() ? "$\\pi$" : "\\pi";
+			}
 		} else {
 			gle_int_to_string((int)floor(num+1e-7), &str);
 			*output += str;
