@@ -248,6 +248,7 @@ void GLEParser::initTokenizer() {
 	multi->setEndToken(';');
 	multi->setEndToken(',');
 	multi->setEndToken(')');
+   multi->setEndToken(']');
 	lang->setMulti(multi);
 }
 
@@ -289,6 +290,30 @@ double GLEParser::evalTokenToDouble() throw(ParserError) {
 	return x;
 }
 
+void GLEParser::evalTokenToString(string* str) throw(ParserError) {
+	GLEPcodeList pc_list;
+	GLEPcode pcode(&pc_list);
+	Tokenizer* tokens = getTokens();
+	string& expr = tokens->next_multilevel_token();
+	int pos = tokens->token_pos_col();
+	try {
+		m_polish->internalEvalString(expr.c_str(), str);
+	} catch (ParserError err) {
+		err.incColumn(pos-1);
+		throw err;
+	}
+}
+
+void GLEParser::evalTokenToFileName(string* str) throw(ParserError) {
+	Tokenizer* tokens = getTokens();
+   const string& token = tokens->next_continuous_string_excluding("\"$+");
+   if (token != "") {
+      *str = token;
+   } else {
+      evalTokenToString(str);
+   }
+}
+
 void GLEParser::polish(GLEPcode& pcode, int *rtype) throw(ParserError) {
 	Tokenizer* tokens = getTokens();
 	string& expr = tokens->next_multilevel_token();
@@ -2320,27 +2345,28 @@ void mark_clear(void) {
 	}
 }
 
-int pass_marker(char *s) throw(ParserError) {
-	int i;
-	int f=0;
+int pass_marker(char *name) throw(ParserError) {
+	int f = 0;
+	string marker;
+	polish_eval_string(name, &marker);
 	/* if 0, maybe its a user defined marker, ie a subroutine */
 	/* Use -ve to signify subroutine instead of normal marker */
-	for (i=0; i<nmark; i++) {
-		if (str_i_equals(mark_name[i],s)) {
+	for (int i = 0; i < nmark; i++) {
+		if (str_i_equals(mark_name[i], marker.c_str())) {
 			f = -(++i);
 			break;
 		}
 	}
-	if (f==0)  {
-		for (i=nmrk-1; i>=0; i--) {
-			if (str_i_equals(mrk_name[i],s)) {
+	if (f == 0)  {
+		for (int i = nmrk-1; i >= 0; i--) {
+			if (str_i_equals(mrk_name[i], marker.c_str())) {
 				f = ++i;
 				break;
 			}
 		}
 	}
 	if (f == 0) {
-		g_throw_parser_error("invalid marker name '", s, "'");
+		g_throw_parser_error("invalid marker name '", marker.c_str(), "'");
 	}
 	return f;
 }
