@@ -237,7 +237,7 @@ void GLEPolish::polish(GLEPcode& pcode, int *rtype) throw(ParserError) {
 				break;
 			}
 			/* Is it a string */
-			if (first_char == '"') {
+			if (first_char == '"' || first_char == '\'') {
 				dbg gprint("Found string \n");
 				last_typ = 2;
 				string str_no_quote = token;
@@ -382,6 +382,23 @@ void GLEPolish::internalEval(const char *exp, double *x) throw(ParserError) {
 	::eval((int*)&pcode[0], &cp, x, NULL, &otyp);
 }
 
+void GLEPolish::internalEvalString(const char* exp, string* str) throw(ParserError) {
+   double oval;
+	const char* ostr;
+	int rtype = 2, otyp = 0, cp = 0;
+	GLEPcodeList pc_list;
+	GLEPcode pcode(&pc_list);
+	polish(exp, pcode, &rtype);
+	::eval((int*)&pcode[0], &cp, &oval, &ostr, &otyp);
+	if (otyp == 1) {
+		stringstream str_cnv;
+		str_cnv << oval;
+		*str = str_cnv.str();
+	} else {
+		*str = ostr;
+	}
+}
+
 void GLEPolish::eval_string(const char *exp, string *str, bool allownum) throw(ParserError) {
 	const char* ostr;
 	double oval = 0.0;
