@@ -1546,7 +1546,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 				GLETextDO drawobj(orig, temp_str);
 				handleNewDrawObject(&drawobj, mkdrobjs);
 			} else {
-				g_text((char*)temp_str.c_str());
+				g_text(temp_str);
 			}
 			g_get_bounds(&x1,&y1,&x2,&y2);
 			break;
@@ -1614,16 +1614,20 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			}
 			if (p == 49) {
 				/* WRITE */
-				if (mkdrobjs) {
-					GLETextDO drawobj(orig, temp_str);
-					handleNewDrawObject(&drawobj, mkdrobjs, &orig);
+				if (g_get_tex_labels()) {
+					TeXInterface::getInstance()->draw(temp_str.c_str());
 				} else {
-					g_text((char*)temp_str.c_str());
+					if (mkdrobjs) {
+						GLETextDO drawobj(orig, temp_str);
+						handleNewDrawObject(&drawobj, mkdrobjs, &orig);
+					} else {
+						g_text(temp_str);
+					}
 				}
 				g_move(orig);
 			} else {
 				/* PRINT */
-				g_message((char*)temp_str.c_str());
+				g_message(temp_str);
 			}
 			break;
 		  case 74: /* TeX */
