@@ -89,7 +89,6 @@ void pass_points();
 void getstr(char *s);
 float getf(void);
 int geton(void);
-FILE *myfopen(const char *fname, const char *mode);
 void pass_data(int *nx, int *ny, float *zmin, float *zmax);
 extern int trace_on,this_line;
 char input_file[80];
@@ -104,6 +103,14 @@ double zclipmin,zclipmax;
 int zclipminset,zclipmaxset;
 int ct;
 
+surface_struct* GLEInterface::getSurface() {
+	return &sf;
+}
+
+void clean_surface() {
+	sf.z = NULL;
+}
+
 void begin_surface(int *pln, int *pcode, int *cp) throw(ParserError) {
 	int li;
 	double ox, oy;
@@ -150,6 +157,11 @@ void begin_surface(int *pln, int *pcode, int *cp) throw(ParserError) {
 		zmin = sf.zaxis.min;
 		zmax = sf.zaxis.max;
 	}
+	sf.nx = nx;
+	sf.ny = ny;
+	sf.z = z;
+	sf.zmin = zmin;
+	sf.zmax = zmax;
 	hide(z,nx,ny,zmin,zmax,&sf);
 	g_move(ox, oy);
 }
@@ -238,7 +250,7 @@ void pass_title() {
 }
 
 void pass_anytitle() {
-        struct axis_struct_3d *ax = NULL;
+        struct GLEAxis3D *ax = NULL;
         if (toupper(*tk[ct])=='X') ax = &sf.xaxis;
         else if (toupper(*tk[ct])=='Y') ax = &sf.yaxis;
         else if (toupper(*tk[ct])=='Z') ax = &sf.zaxis;
@@ -366,7 +378,7 @@ void pass_marker() {
 }
 
 void pass_axis() {
-        struct axis_struct_3d *ax=NULL;
+        struct GLEAxis3D *ax=NULL;
         if (toupper(*tk[ct])=='X') ax = &sf.xaxis;
         if (toupper(*tk[ct])=='Y') ax = &sf.yaxis;
         if (toupper(*tk[ct])=='Z') ax = &sf.zaxis;
@@ -390,7 +402,6 @@ void pass_axis() {
 }
 
 static char buff[2032];
-FILE *myfopen(char *fname, char *mode);
 
 bool alloc_zdata(int nx, int ny) {
         if (z!=NULL) free(z);
@@ -436,8 +447,7 @@ void pass_data(int *nx, int *ny, float *zmin, float *zmax) {
         if (nx==NULL || ny==0) printf("nx or ny is zero \n");
         if (*nx!=0 && *ny != 0)   if (alloc_zdata(*nx,*ny)) return;
 
-	validate_file_name(fname, true);
-        df = myfopen(fname.c_str(),"r");
+        df = validate_fopen(fname.c_str(), "r", true);
         if (df==NULL) {*nx = 0; *ny = 0; return;}
         for (;!feof(df);) {
           if (fgets(buff,2000,df)!=NULL) {
@@ -531,8 +541,7 @@ void pass_points() {
                 return;
         }
 
-	validate_file_name(fname, true);
-        df = myfopen(fname.c_str(),"r");
+        df = validate_fopen(fname.c_str(), "r", true);
         if (df==NULL) return;
         nd = 0;
         for (;!feof(df);) {
@@ -601,13 +610,3 @@ void hide_defaults() {
         sf.bot_on = false;
         sf.base_hidden = sf.right_hidden = sf.back_hidden = true;
 }
-
-FILE *myfopen(const char *fname, const char *mode) {
-        FILE *f;
-        f = fopen(fname,mode);
-        if (f==NULL) {
-                gprint("Unable to open {%s} \n\n",fname); perror("");
-                return NULL;
-        }
-        return f;
-}
