@@ -59,8 +59,8 @@ void skirt(float huge *z,int ix1, int iy1, float minz);
 void find_splits(int nx, int ny, int *splitx, int *splity);
 void fxy_polar(float dx,float dy,float *radius,float *angle);
 void fpolar_xy(float r, float angle, float *dx, float *dy);
-void draw_zaxis(struct axis_struct_3d *ax, int nx, int ny, float minz, float maxz);
-void draw_axis(struct axis_struct_3d *ax, int nx, int ny, float minz, float maxz);
+void draw_zaxis(struct GLEAxis3D *ax, int nx, int ny, float minz, float maxz);
+void draw_axis(struct GLEAxis3D *ax, int nx, int ny, float minz, float maxz);
 void draw_maintitle();
 void hide(float huge *z,int nx, int ny, float minz, float maxz, struct surface_struct *sff);
 void touser3(float x, float y, float z, float *uux, float *uuy, float *uuz);
@@ -135,6 +135,34 @@ int vsign=1;
 #define returng {v_close(); return;}
 static struct surface_struct sf;
 
+float get_h(int i) {
+	if (i < 0 || i >= MAXH) {
+		return 0.0;
+	} else {
+		return h[i];
+	}
+}
+
+void set_h(int i, float v) {
+	if (!(i < 0 || i >= MAXH)) {
+		h[i] = v;
+	}
+}
+
+float get_h2(int i) {
+	if (i < 0 || i >= MAXH) {
+		return GLE_INF;
+	} else {
+		return h2[i];
+	}
+}
+
+void set_h2(int i, float v) {
+	if (!(i < 0 || i >= MAXH)) {
+		h2[i] = v;
+	}
+}
+
 void hide(float huge *z,int nx, int ny, float minz, float maxz, struct surface_struct *sff) {
 	int i,x,y;
 	float r,angle;
@@ -165,8 +193,8 @@ void hide(float huge *z,int nx, int ny, float minz, float maxz, struct surface_s
 	MAXH = sf.maxh;
 
 	if (MAXH==0) MAXH = 5000;
-	h = (float*)malloc(MAXH*sizeof(float));
-	h2 = (float*)malloc(MAXH*sizeof(float));
+	h = (float*)malloc((MAXH+1)*sizeof(float));
+	h2 = (float*)malloc((MAXH+1)*sizeof(float));
 	if (h==NULL || h2==NULL) {
 		gprint("Was not able to allocate horizon arrays %d \n",MAXH);
 		return;
@@ -409,7 +437,7 @@ void draw_maintitle() {
 	v_text((char*)sf.title);
 }
 
-void draw_axis(struct axis_struct_3d *ax, int nx, int ny, float minz, float maxz) {
+void draw_axis(struct GLEAxis3D *ax, int nx, int ny, float minz, float maxz) {
 	float ux,uy,ux2,uy2,ux3,uy3,r,a,ta,r2,x,xx,t1,tn;
 	char buff[80];
 	if (ax->type>1) return;
@@ -479,7 +507,7 @@ void draw_axis(struct axis_struct_3d *ax, int nx, int ny, float minz, float maxz
 	g_grestore();
 }
 
-void draw_zaxis(struct axis_struct_3d *ax, int nx, int ny, float minz, float maxz) {
+void draw_zaxis(struct GLEAxis3D *ax, int nx, int ny, float minz, float maxz) {
 	float ux,uy,ux2,uy2,ux3,uy3,r,a,ta,r2,x,t1,tn;
 	char buff[80];
 
@@ -544,14 +572,14 @@ void line3d(float x, float y, float z) {
 void show_horizon() {
 	int i;
 	v_color("RED");
-	v_move(unmaph(0),h[0]);
+	v_move(unmaph(0), get_h(0));
 	for (i=0;i<900;i++) {
-		v_line(unmaph(i),h[i]);
+		v_line(unmaph(i), get_h(i));
 	}
 	v_color("BLUE");
-	v_move(unmaph(0),h2[0]);
+	v_move(unmaph(0), get_h2(0));
 	for (i=0;i<900;i++) {
-		v_line(unmaph(i),h2[i]);
+		v_line(unmaph(i), get_h2(i));
 	}
 
 }
@@ -628,11 +656,11 @@ void hclipvec(int x1, float y1, int x2, float y2, int sethi) {
 	int i,sd,sx = 0;
 	if (x1==x2) {
 		/* printf("VEC %d %g   %d %g h=%g\n",x1,y1,x2,y2,h[x1]); scr_getch();  */
-        if (y2<y1) {v = y1; y1 = y2; y2 = v;}
-		if (h[x1]<y2) {
-			if (h[x1]>y1) y1 = h[x1];
+		if (y2<y1) {v = y1; y1 = y2; y2 = v;}
+		if (get_h(x1) < y2) {
+			if (get_h(x1) > y1) y1 = get_h(x1);
 			vector_line(x1,y1,x2,y2);
-			if (sethi) h[x1] = y2;
+			if (sethi) set_h(x1, y2);
 		}
 		return;
 	}
@@ -644,15 +672,15 @@ void hclipvec(int x1, float y1, int x2, float y2, int sethi) {
 	for (i=x1, v=y1; sd*i <= sd*x2; i+=sd, v+=step) {
 		if (visible) {
 			// FIXME: add range check for h-array?
-			if (h[i]>v) {
+			if (get_h(i) > v) {
 				if (sethi) vector_line(sx,sy,i-sd,v-step);
 				else vector_line(sx,sy,i-sd,v-step);
 				visible = false;
-			} else if (sethi) h[i] = v;
+			} else if (sethi) set_h(i, v);
 		} else {
-			if (h[i]<=v+.0001) {
+			if (get_h(i) <= v+.0001) {
 				sx = i; sy = v; visible = true;
-				if (sethi) h[i] = v;
+				if (sethi) set_h(i, v);
 			}
 		}
 	}
@@ -678,7 +706,7 @@ void seth2(int rx1, int ry1, float rz1, int rx2, int ry2, float rz2) {
 
        if (x1==x2) {
 		if (y2>y1) {v = y1; y1 = y2; y2 = v;}
-		if (h2[x1]>y2) h2[x1] = y2;
+		if (get_h2(x1) > y2) set_h2(x1, y2);
 		return;
        }
        step = (y2-y1)/(x2-x1);
@@ -687,46 +715,61 @@ void seth2(int rx1, int ry1, float rz1, int rx2, int ry2, float rz2) {
        step = step*sd;
        visible = false;
        for (i=x1, v=y1; sd*i <= sd*x2; i+=sd, v+=step) {
-		if (h2[i]>v) h2[i] = v;
+		if (get_h2(i) > v) set_h2(i, v);
        }
 }
 
-void hclipvec2(int x1, float y1, int x2, float y2, int sethi) {
-	float v,sy = 0,step;
-	int i,sd,sx = 0,visible;
+void vector_line_d(double x1, double y1, double x2, double y2) {
+	v_move(unmaph(x1),y1);
+	v_line(unmaph(x2),y2);
+}
 
-       if (x1==x2) {
-/*        printf("VEC2 %d %g   %d %g h=%g ",x1,y1,x2,y2,h2[x1]); scr_getch();*/
-		if (y2>y1) {v = y1; y1 = y2; y2 = v;}
-		if (h2[x1]>y2) {
-			if (h2[x1]<y1) y1 = h2[x1];
+void hclipvec2(int x1, float y1, int x2, float y2, int sethi) {
+	if (x1 == x2) {
+		if (y2 > y1) {
+			double v = y1;
+			y1 = y2;
+			y2 = v;
+		}
+		if (get_h2(x1) > y2) {
+			if (get_h2(x1) < y1) y1 = get_h2(x1);
 			vector_line(x1,y1,x2,y2);
-			if (sethi) h2[x1] = y2;
+			if (sethi) set_h2(x1, y2);
 		}
-/*                scr_getch(); printf("x \n"); scr_getch();                        */
 		return;
-       }
-       step = (y2-y1)/(x2-x1);
-       sd = -1;
-       if (x1<x2) sd = 1;
-       step = step*sd;
-       visible = false;
-       for (i=x1, v=y1; sd*i <= sd*x2; i+=sd, v+=step) {
+	}
+	GLELinearEquation vec, horiz;
+	vec.fit(x1, y1, x2, y2);
+	int sx = 0;
+	int sd = (x1 < x2) ? 1 : -1;
+	bool visible = false;
+	for (int i = x1; sd*i <= sd*x2; i+=sd) {
+		double v = vec.apply(i);
 		if (visible) {
-			if (h2[i]<v) {
-				if (sethi)  vector_line(sx,sy,i-sd,v-step);
-				else  vector_line(sx,sy,i-sd,v-step);
+			if (get_h2(i) < v) {
+				// drops below horizon -> invisible
 				visible = false;
-			} else if (sethi) h2[i] = v;
+				// find exact point from where on invisible
+				horiz.fit(i-sd, get_h2(i-sd), i, get_h2(i));
+				double xp = vec.intersect(&horiz);
+				vector_line_d(sx, vec.apply(sx), xp, vec.apply(xp));
+			} else if (sethi) set_h2(i, v);
 		} else {
-			if (h2[i]>=v-.0001) {
-				sx = i; sy = v; visible = true;
-				if (sethi) h2[i] = v;
+			if (get_h2(i) >= v-.0001) {
+				// back above horizon -> again visible
+				visible = true;
+				if (i == x1) {
+					sx = i;
+				} else {
+					horiz.fit(i-sd, get_h2(i-sd), i, get_h2(i));
+					sx = vec.intersect(&horiz);
+				}
+				if (sethi) set_h2(i, v);
 			}
 		}
-       }
-       /* draw end part of line */
-       if (visible) vector_line(sx,sy,x2,y2);
+	}
+	/* draw end part of line */
+	if (visible) vector_line(sx, vec.apply(sx), x2, y2);
 }
 
 void vector_line(int x1, float y1, int x2, float y2) {
