@@ -70,7 +70,9 @@
  *    fftext_block
  *        text_tomacro
  *        text_topcode
+ *           uses: do_prim
  *        text_wrapcode
+ *           uses: set_glue
  *        text_draw
  *
  * text_tomacro
@@ -421,6 +423,7 @@ uchar* cmdParam(uchar **inp, char** pm, int* pmlen, int npm) {
 	return save_inp;
 }
 
+// used by topcode()
 void text_box(const string& s,double width,int *tbuff, int *rplen) {
 	int plen=0;
 	char *workbuff;
@@ -922,7 +925,7 @@ void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params) {
 	  case tp_tex:
 		params->cmdParam1(in);
 		outlong(11);
-		outlong(TeXInterface::getInstance()->createObj(params->getCStr1()));
+		outlong(TeXInterface::getInstance()->createObj(params->getCStr1(), p_hei));
 	  	break;
 	  case tp_frac:
 	  case tp_delimiter:
