@@ -129,7 +129,7 @@ TeXObject* TeXInterface::draw(const char* str, TeXObjectInfo& info, int nblines,
 	string obj_str = str;
 	/* replace \'' by \" so that umlauts work in LaTeX expressions */
 	str_replace_all(obj_str, "\\''", "\\\"");
-	scaleObject(obj_str);
+	scaleObject(obj_str, 0.0);
 	TeXHashObject* hobj = getHashObject(obj_str);
 	hobj->setNbLines(nblines);
 	hobj->setUsed(true);
@@ -137,13 +137,12 @@ TeXObject* TeXInterface::draw(const char* str, TeXObjectInfo& info, int nblines,
 	return drawObj(hobj, info, box);
 }
 
-void TeXInterface::scaleObject(string& obj_str) {
+void TeXInterface::scaleObject(string& obj_str, double hei) {
 	int scaleMode = getScaleMode();
 	if (scaleMode != TEX_SCALE_MODE_NONE) {
-		double hei;
 		TeXPreambleInfo* preamble = getCurrentPreamble();
 		if (!preamble->hasFontSizes()) checkTeXFontSizes();
-		g_get_hei(&hei);
+		if (hei == 0) g_get_hei(&hei);
 		if (scaleMode == TEX_SCALE_MODE_FIXED) {
 			int best_size = preamble->getBestSizeFixed(hei);
 			if (best_size != -1) {
@@ -250,14 +249,14 @@ void TeXInterface::checkObjectDimensions() {
 	}
 }
 
-int TeXInterface::createObj(const char* str) {
+int TeXInterface::createObj(const char* str, double hei) {
 	/* Load hash */
 	tryLoadHash();
 	/* Create object given string */
 	string obj_str = str;
 	/* replace \'' by \" so that umlauts work in LaTeX expressions */
 	str_replace_all(obj_str, "\\''", "\\\"");
-	scaleObject(obj_str);
+	scaleObject(obj_str, hei);
 	int result = getHashObjectIndex(obj_str);
 	m_TeXHash[result]->setUsed(true);
 	return result;
@@ -1029,54 +1028,67 @@ void TeXObjectInfo::initializeAll() {
 	}
 }
 
-bool adjust_bounding_box(const string& name, double* width, double* height, ostream& out) {
+bool read_eps_and_adjust_bounding_box(const string& name, GLEScript* script) {
 	int b1 = 0, b2 = 0, b3 = 0, b4 = 0;
-	string in_fname = name + ".eps";
-	/* Find bounding box */
-	StreamTokenizerMax tokens(in_fname,' ', 50);
-	while (tokens.hasMoreTokens()) {
-		const char* token = tokens.nextToken();
-		if (str_i_str(token, "%%BoundingBox") != NULL) {
-			b1 = tokens.hasMoreTokens() ? atoi(tokens.nextToken()) : 0;
-			b2 = tokens.hasMoreTokens() ? atoi(tokens.nextToken()) : 0;
-			b3 = tokens.hasMoreTokens() ? atoi(tokens.nextToken()) : 0;
-			b4 = tokens.hasMoreTokens() ? atoi(tokens.nextToken()) : 0;
+	string fileName(name + ".eps");
+	vector<string> lines;
+	if (!GLEReadFile(fileName, &lines)) {
+		return false;
+	}
+	unsigned int pos = 0;
+	ostringstream out;
+	while (pos < lines.size()) {
+		string line(lines[pos++]);
+		if (g_parse_ps_boundingbox(line, &b1, &b2, &b3, &b4))
+		{
+			time_t t;
+			t = time(0);
+			GLEPoint bb(script->getBoundingBox());
+			// Make sure to use proper Adobe DSC comments
+			// Windows Explorer can show these in the document properties of a PDF created by GLE
+			string vers_nosnap = g_get_version_nosnapshot();
+			out << "%%Creator: GLE " << vers_nosnap << " <www.gle-graphics.org>" << endl;
+			out << "%%CreationDate: " << ctime(&t);
+			out << "%%Title: " << script->getLocation()->getName() << endl;
+			int b3New = (int)ceil(b1 + bb.getX() + 1e-6);
+			int b4New = (int)ceil(b2 + bb.getY() + 1e-6);
+			out << "%%BoundingBox: " << b1 << " " << b2 << " " << b3New << " " << b4New << endl;
+			script->setBoundingBoxOrigin(b1, b2);
 		}
-		if (str_i_str(token, "EndComments")) {
+		else if (str_starts_with_trim(line, "%%HiResBoundingBox") != -1 ||
+		         str_starts_with_trim(line, "%%Creator") != -1 ||
+		         str_starts_with_trim(line, "%%CreationDate") != -1 ||
+		         str_starts_with_trim(line, "%%Title") != -1)
+		{
+			// Forget about these
+		}
+		else if (str_starts_with_trim(line, "%%EndComments") != -1)
+		{
+			out << line << endl;
 			break;
 		}
-	}
-	if (b1 != 0 || b2 != 0 || b3 != 0 || b4 != 0) {
-		out << "%!PS-Adobe-2.0 EPSF-2.0" << endl;
-		out << "%%BoundingBox: 0 0 " << (b3-b1) << " " << (b4-b2) << endl;
-		if (b1 != 0 || b2 != 0) {
-			out << "gsave " << (-b1) << " " << (-b2) << " translate" << endl;
-			*width = (double)b3-b1+1;
-			*height = (double)b4-b2+1;
-		}
-		ifstream& strm = tokens.getFile();
-		GLECopyStream(strm, out);
-		if (b1 != 0 || b2 != 0) {
-			out << endl << "grestore" << endl;
+		else
+		{
+			out << line << endl;
 		}
 	}
-	tokens.close();
+	while (pos < lines.size()) {
+		string line(lines[pos++]);
+		out << line << endl;
+	}
+	string* buffer = script->getRecordedBytesBuffer(GLE_DEVICE_EPS);
+	*buffer = out.str();
 	return true;
 }
 
-bool create_bitmap_file(GLEFileLocation* fname, istream* epss, int device, int dpi, const GLEPoint& bb, bool bw, bool transp, bool tex) {
+bool create_bitmap_file(GLEFileLocation* fname, int device, int dpi, bool bw, bool transp, GLEScript* script) {
 	ostringstream gsargs;
-	stringstream adjusted_ps;
 	gsargs << "-q -DNOPLATFONTS -dTextAlphaBits=4 -dGraphicsAlphaBits=4 -dBATCH -dNOPAUSE -r";
 	gsargs << dpi;
-	double width = bb.getX();
-	double height = bb.getY();
-	if (tex) {
-		adjust_bounding_box(fname->getFullPath(), &width, &height, adjusted_ps);
-		epss = &adjusted_ps;
-	}
-	int img_wd = GLEBBoxToPixels(dpi, width);
-	int img_hi = GLEBBoxToPixels(dpi, height);
+	GLEPoint bb(script->getBoundingBox());
+	GLEPoint origin(script->getBoundingBoxOrigin());
+	int img_wd = GLEBBoxToPixels(dpi, bb.getX());
+	int img_hi = GLEBBoxToPixels(dpi, bb.getY());
 	gsargs << " -g" << img_wd << "x" << img_hi;
 	string gs_opts = g_CmdLine.getOptionString(GLE_OPT_GSOPTIONS);
 	if (gs_opts != "") {
@@ -1107,16 +1119,16 @@ bool create_bitmap_file(GLEFileLocation* fname, istream* epss, int device, int d
 		}
 		gsargs << " -sOutputFile=\"" << outputfile << "\"";
 	}
-	if (epss != NULL) {
-		gsargs << " -";
-	} else {
-		gsargs << " \"" << fname->getFullPath() << ".eps\"";
-	}
-	return run_ghostscript(gsargs.str(), outputfile, !fname->isStdout(), epss);
+	gsargs << " -";
+	stringstream postscript;
+	string* bytes = script->getRecordedBytesBuffer(GLE_DEVICE_EPS);
+	postscript << (-origin.getX()) << " " << (-origin.getY()) << " translate" << endl;
+	postscript.write(bytes->data(), bytes->size());
+	return run_ghostscript(gsargs.str(), outputfile, !fname->isStdout(), &postscript);
 }
 
-bool create_pdf_file_ghostscript(GLEFileLocation* fname, istream* epss, int dpi, const GLEPoint& bb, bool tex) {
-	stringstream gsargs;
+bool create_pdf_file_ghostscript(GLEFileLocation* fname, int dpi, GLEScript* script) {
+	ostringstream gsargs;
 	gsargs << "-q";
 	int compr_mode = g_get_pdf_image_format();
 	switch (compr_mode) {
@@ -1152,6 +1164,8 @@ bool create_pdf_file_ghostscript(GLEFileLocation* fname, istream* epss, int dpi,
 			gsargs << " -dEncodeMonoImages=false"; break;
 	}
 	gsargs << " -dBATCH -dNOPAUSE -r" << dpi;
+	GLEPoint bb(script->getBoundingBox());
+	GLEPoint origin(script->getBoundingBoxOrigin());
 	int img_wd = GLEBBoxToPixels(dpi, bb.getX());
 	int img_hi = GLEBBoxToPixels(dpi, bb.getY());
 	gsargs << " -g" << img_wd << "x" << img_hi;
@@ -1163,18 +1177,12 @@ bool create_pdf_file_ghostscript(GLEFileLocation* fname, istream* epss, int dpi,
 		outputfile = fname->getFullPath() + ".pdf";
 		gsargs << " -sOutputFile=\"" << outputfile << "\"";
 	}
-	stringstream adjusted_ps;
-	if (epss != NULL || tex) {
-		if (tex) {
-			double dummy_w, dummy_h;
-			adjust_bounding_box(fname->getFullPath(), &dummy_w, &dummy_h, adjusted_ps);
-			epss = &adjusted_ps;
-		}
-		gsargs << " -";
-	} else {
-		gsargs << " \"" << fname->getFullPath() << ".eps\"";
-	}
-	return run_ghostscript(gsargs.str(), outputfile, !fname->isStdout(), epss);
+	gsargs << " -";
+	stringstream postscript;
+	string* bytes = script->getRecordedBytesBuffer(GLE_DEVICE_EPS);
+	postscript << (-origin.getX()) << " " << (-origin.getY()) << " translate" << endl;
+	postscript.write(bytes->data(), bytes->size());
+	return run_ghostscript(gsargs.str(), outputfile, !fname->isStdout(), &postscript);
 }
 
 void report_latex_errors_parse_error(istream& strm, string* result) {
@@ -1309,21 +1317,22 @@ bool run_latex(const string& dir, const string& file) {
 	return result_ok;
 }
 
-bool create_eps_file_latex_dvips(const string& fname) {
+bool create_eps_file_latex_dvips(const string& fname, GLEScript* script) {
 	string file, dir;
 	ConfigSection* tex = g_Config.getSection(GLE_CONFIG_TEX);
 	CmdLineArgSet* texsys =	(CmdLineArgSet*)tex->getOptionValue(GLE_TEX_SYSTEM);
 	SplitFileName(fname, dir, file);
 	if (!run_latex(dir, file)) return false;
 	if (!run_dvips(fname, true)) return false;
+	bool result = read_eps_and_adjust_bounding_box(fname, script);
 	DeleteFileWithExt(fname, ".aux");
 	if (texsys->hasValue(GLE_TEX_SYSTEM_VTEX)) {
-	DeleteFileWithExt(fname, ".ps");
+		DeleteFileWithExt(fname, ".ps");
 	} else {
 		DeleteFileWithExt(fname, ".dvi");
 	}
 	DeleteFileWithExt(fname, ".log");
-	return true;
+	return result;
 }
 
 bool create_ps_file_latex_dvips(const string& fname) {
@@ -1403,13 +1412,10 @@ bool run_dvips(const string& file, bool eps) {
 		// use ghostscript to convert PS to EPS
 		if (eps) {
 			string gsargs;
-			string outputfile = file + ".eps";
+			string outputfile(file + ".eps");
 			gsargs += "-dNOPAUSE -sDEVICE=epswrite -sOutputFile=";
 			gsargs += outputfile;
-			// for one-page PS files use:
 			gsargs += " -q -sBATCH \"";
-			// for multi-page PS files use (%03d -> 001,002,...):
-			// gsargs += ".%03d -q -sBATCH ";
 			gsargs += file;
 			gsargs += ".ps\"";
 			return run_ghostscript(gsargs, outputfile);
@@ -1427,7 +1433,7 @@ bool run_dvips(const string& file, bool eps) {
 		const string dvips_opts(tools->getOptionString(GLE_TOOL_DVIPS_OPTIONS));
 		if (!dvips_opts.empty()) dvipsargs << " " << dvips_opts;
 		if (eps) dvipsargs << " -E";
-		const string outfile = file + (eps ? ".eps" : ".ps");
+		const string outfile(file + (eps ? ".eps" : ".ps"));
 		dvipsargs << " -o \"" << outfile << "\" \"" << file << ".dvi\"";
 		string cmdline = dvipsargs.str();
 		if (g_verbosity() >= 5) {
