@@ -225,9 +225,9 @@ public:
 	TeXObject* draw(const char* str, int nblines, GLERectangle* box = NULL) throw(ParserError);
 	TeXObject* draw(const char* str, TeXObjectInfo& info, int nblines, GLERectangle* box = NULL) throw(ParserError);
 	TeXObject* drawObj(TeXHashObject* hobj, TeXObjectInfo& info, GLERectangle* box = NULL) throw(ParserError);
-	void scaleObject(string& obj_str);
+	void scaleObject(string& obj_str, double hei);
 	void checkObjectDimensions();
-	int createObj(const char* str);
+	int createObj(const char* str, double hei);
 	void initialize(GLEFileLocation* dotfile, GLEFileLocation* oname);
 	void updateNames(GLEFileLocation* dotfile, GLEFileLocation* oname);
 	void updateOutName(GLEFileLocation* oname);
@@ -273,12 +273,12 @@ protected:
 	void retrieveTeXFontSizes(TeXHash& tex_hash, TeXPreambleInfo* preamble);
 };
 
-bool create_eps_file_latex_dvips(const string& fname);
+bool create_eps_file_latex_dvips(const string& fname, GLEScript* script);
 bool create_ps_file_latex_dvips(const string& fname);
 bool create_pdf_file_pdflatex(const string& fname);
 
-bool create_bitmap_file(GLEFileLocation* fname, istream* epss, int device, int dpi, const GLEPoint& bb, bool bw, bool transp, bool tex);
-bool create_pdf_file_ghostscript(GLEFileLocation* fname, istream* epss, int dpi, const GLEPoint& bb, bool tex);
+bool create_bitmap_file(GLEFileLocation* fname, int device, int dpi, bool bw, bool transp, GLEScript* script);
+bool create_pdf_file_ghostscript(GLEFileLocation* fname, int dpi, GLEScript* script);
 
 bool run_ghostscript(const string& args, const string& outfile, bool redirout = true, istream* is = NULL);
 bool run_latex(const string& dir, const string& file);
