@@ -626,12 +626,11 @@ string& Tokenizer::try_next_token() throw(ParserError) {
 }
 
 double Tokenizer::next_double() throw(ParserError) {
-	char *endptr;
+	char *ptr;
 	get_check_token();
-	if (!is_float(m_token)) {
-		throw error("expected floating point number, not '" + m_token + "'");
-	}
-	return strtod(m_token.c_str(), &endptr);
+	double result = strtod(m_token.c_str(), &ptr);
+   if (*ptr != 0) throw error("expected floating point number, not '" + m_token + "'");
+	return result;
 }
 
 int Tokenizer::next_integer() throw(ParserError) {
@@ -700,48 +699,20 @@ void Tokenizer::get_token_2() throw(ParserError) {
 		 m_token = "";
 		 return;
 	}
-	// String with first type of marks \'
-	if (token_ch == '\'' && m_language->getParseStrings()) {
-		char p1_ch = 0, p2_ch = 0;
+	// String with \" or \' marks
+	if ((token_ch  == '\"' || token_ch == '\'') && m_language->getParseStrings()) {
+      char string_delim = token_ch;
 		m_token = token_ch;
 		do {
 			token_ch = token_read_char_no_comment();
-			m_token += token_ch;
-			// Support ''', '\'' and '\\' allowed by some prologs
-			if (token_ch == '\'' && (p1_ch != '\\' || p2_ch == '\\')) {
+         m_token += token_ch;
+			if (token_ch == string_delim) {
 				token_ch = token_read_char_no_comment();
-				if (token_ch == '\'') {
-					m_token += token_ch;
-				} else {
+				if (token_ch != string_delim) {
 					token_pushback_ch(token_ch);
 					return;
 				}
-			}
-			p2_ch = p1_ch;
-			p1_ch = token_ch;
-		} while (m_token_at_end == 0);
-		throw error("unterminated string constant");
-	}
-	// String with second type of marks \"
-	if (token_ch  == '\"' && m_language->getParseStrings()) {
-		int prev_backslash = 0;
-		m_token = token_ch;
-		do {
-			token_ch = token_read_char_no_comment();
-			if (token_ch == '\"') {
-				// Quote character found - ok if preceeded by *even* number of backslashes
-				if (prev_backslash % 2 == 0) {
-					m_token += token_ch;
-					return;
-				} else {
-					// Erase backslash and replace by '"'
-					m_token[m_token.size()-1] = token_ch;
-				}
-			} else {
-				m_token += token_ch;
-			}
-			if (token_ch == '\\') prev_backslash++;
-			else prev_backslash = 0;
+         }
 		} while (m_token_at_end == 0);
 		throw error("unterminated string constant");
 	}
@@ -842,7 +813,33 @@ void Tokenizer::multi_level_do_multi(char open) throw(ParserError) {
 	}
 }
 
-string& Tokenizer::next_multilevel_token() throw(ParserError) {
+string& Tokenizer::next_continuous_string_excluding(const char* forbidden) throw(ParserError) {
+   undo_pushback_token();
+	m_token = "";
+	char token_ch = token_read_sig_char();
+	m_token_start = m_token_count;
+	if (m_token_at_end == 1) {
+		return m_token;
+	}
+	do {
+		if (token_ch == SPACE_CHAR) {
+         break;
+		} else {
+         if (str_contains(forbidden, token_ch)) {
+            m_token = "";
+            goto_position(m_token_start);
+            break;
+         } else {
+            m_token += token_ch;
+            token_ch = token_read_char();
+         }
+		}
+	} while (m_token_at_end == 0);
+	return m_token;
+}
+
+void Tokenizer::undo_pushback_token()
+{
 	if (m_token_has_pushback > 0) {
 		TokenAndPos& tkpos = m_pushback_tokens.back();
 		// cout << "goto position = " << tkpos.getPos() << endl;
@@ -850,6 +847,10 @@ string& Tokenizer::next_multilevel_token() throw(ParserError) {
 		m_pushback_tokens.clear();
 		m_token_has_pushback = 0;
 	}
+}
+
+string& Tokenizer::next_multilevel_token() throw(ParserError) {
+   undo_pushback_token();
 	m_token = "";
 	char token_ch = token_read_sig_char();
 	m_token_start = m_token_count;
@@ -1254,6 +1255,12 @@ void Tokenizer::read_till_close_comment() throw(ParserError) {
 	}
 }
 
+ParserError Tokenizer::error(const char* s1, const char* s2, const char* s3) const {
+   ostringstream err;
+   err << s1 << s2 << s3;
+   return error(err.str());
+}
+
 ParserError Tokenizer::error(const string& src) const {
 	ParserError err(src, token_pos(), m_fname);
 	const char* parse_str = parse_string_in_error();
