@@ -298,6 +298,8 @@ public:
 	string& next_token() throw(ParserError);
 	// returns next token.  throws exception if no more tokens.
 
+   string& next_continuous_string_excluding(const char* forbidden) throw(ParserError);
+
 	string& try_next_token() throw(ParserError);
 
 	string& read_line() throw(ParserError);
@@ -415,6 +417,8 @@ public:
 
 	ParserError error(const string& src) const;
 
+	ParserError error(const char* s1, const char* s2, const char* s3) const;
+
 	ParserError error(const TokenizerPos& pos, const string& src) const;
 
 	ParserError error(int column, const string& src) const;
@@ -424,6 +428,8 @@ public:
 	virtual const char* parse_string_in_error() const;
 
 protected:
+   void undo_pushback_token();
+
 	void copy_string(char endch) throw(ParserError);
 
 	void multi_level_do_multi(char open) throw(ParserError);
