@@ -76,26 +76,25 @@ void EvaluatorDialog::clearResults()
 	results->clear();
 }
 
-#define EXP_FMT_ROW_PDF 0
-#define EXP_FMT_ROW_EPS 1
-#define EXP_FMT_ROW_PS  2
-#define EXP_FMT_ROW_SVG 3
-#define EXP_FMT_ROW_PNG 4
-#define EXP_FMT_ROW_JPG 5
-
 ExportDialogue::ExportDialogue(GLEMainWindow *parent) {
 	device = -1;
 	mainWin = parent;
 	setWindowTitle(tr("Export File"));
 	QVBoxLayout *layout = new QVBoxLayout();
 	layout->addWidget(new QLabel(tr("Export to:")));
-	QListWidget* list = new QListWidget();
-	list->addItem(tr("Portable Document Format (PDF)"));
-	list->addItem(tr("Encapsulated PostScript (EPS)"));
-	list->addItem(tr("PostScript Document (PS)"));
-	list->addItem(tr("Scalable Vector Graphics (SVG)"));
-	list->addItem(tr("Portable Network Graphics (PNG)"));
-	list->addItem(tr("JPEG Interchange Format (JPEG)"));
+	list = new QListWidget();
+	GLEInterface* iface = mainWin->getGLEInterface();
+	addFormat(tr("Portable Document Format (PDF)"), GLE_DEVICE_PDF);
+	addFormat(tr("Encapsulated PostScript (EPS)"), GLE_DEVICE_EPS);
+	addFormat(tr("PostScript Document (PS)"), GLE_DEVICE_PS);
+	if (iface->isDeviceSupported(GLE_DEVICE_SVG)) {
+		addFormat(tr("Scalable Vector Graphics (SVG)"), GLE_DEVICE_SVG);
+	}
+	if (iface->isDeviceSupported(GLE_DEVICE_EMF)) {
+		addFormat(tr("Enhanced Metafile (EMF)"), GLE_DEVICE_EMF);
+	}
+	addFormat(tr("Portable Network Graphics (PNG)"), GLE_DEVICE_PNG);
+	addFormat(tr("JPEG Interchange Format (JPEG)"), GLE_DEVICE_JPEG);
 	list->setCurrentRow(mainWin->settings->getExportFormat());
 	list->adjustSize();
 	connect(list, SIGNAL(currentRowChanged(int)), this, SLOT(exportFormatChanged(int)));
@@ -155,13 +154,14 @@ ExportDialogue::ExportDialogue(GLEMainWindow *parent) {
 	exportFormatChanged(mainWin->settings->getExportFormat());
 }
 
+void ExportDialogue::addFormat(const QString& name, int gleFormatID) {
+	list->addItem(name);
+	fmtToDevice.append(gleFormatID);
+}
+
 int ExportDialogue::entryIDtoDevice(int id) {
-	switch (id) {
-		case EXP_FMT_ROW_PDF: return GLE_DEVICE_PDF;
-		case EXP_FMT_ROW_PS:  return GLE_DEVICE_PS;
-		case EXP_FMT_ROW_SVG:  return GLE_DEVICE_SVG;
-		case EXP_FMT_ROW_PNG: return GLE_DEVICE_PNG;
-		case EXP_FMT_ROW_JPG: return GLE_DEVICE_JPEG;
+	if (id < fmtToDevice.size()) {
+		return fmtToDevice[id];
 	}
 	return GLE_DEVICE_EPS;
 }
@@ -285,7 +285,7 @@ SoftwareLocateDialogue::SoftwareLocateDialogue(GLEMainWindow *parent, GLEInterfa
 	QHBoxLayout *top = new QHBoxLayout();
 	QVBoxLayout *left = new QVBoxLayout();
 	QLabel* icon = new QLabel(this);
-	icon->setPixmap(QPixmap(":/images/gle.png").scaled(QSize(64, 64)));
+	icon->setPixmap(QPixmap(":/images/gle_shadow.png").scaled(QSize(64, 64)));
 	left->addWidget(icon);
 	left->addStretch(1);
 	top->addLayout(left);
@@ -454,7 +454,7 @@ CrashRecoverDialogue::CrashRecoverDialogue(const QString& file) : QDialog(NULL)
 	http = NULL;
 	crashLog = file;
 	requestID = -1;
-	setWindowIcon(QIcon(":images/gle.png"));
+	setWindowIcon(QIcon(":images/gle_shadow.png"));
 	setWindowTitle(tr("GLE Internal Error"));
 	crashLogLines.append(QString("The following is the error code and memory location of the error."));
 	crashLogLines.append(QString("We use this to find the line in GLE's source code where the error occured."));
@@ -478,7 +478,7 @@ CrashRecoverDialogue::CrashRecoverDialogue(const QString& file) : QDialog(NULL)
 	QHBoxLayout *top = new QHBoxLayout();
 	QVBoxLayout *left = new QVBoxLayout();
 	QLabel* icon = new QLabel(this);
-	icon->setPixmap(QPixmap(":/images/gle.png").scaled(QSize(64, 64)));
+	icon->setPixmap(QPixmap(":/images/gle_shadow.png").scaled(QSize(64, 64)));
 	left->addWidget(icon);
 	left->addStretch(1);
 	top->addLayout(left);
