@@ -58,9 +58,11 @@ private slots:
 private:
 	void updateResolution();
 	void performExport(const QString& file);
-
+	void addFormat(const QString& name, int gleFormatID);
 	int device;
+	QList<int> fmtToDevice;
 	GLEMainWindow *mainWin;
+	QListWidget* list;
 	QCheckBox* transp;
 	QCheckBox* openResult;
 	QLineEdit* fname;
