@@ -93,12 +93,30 @@ typedef BOOL (__stdcall *pStackWalk)(DWORD MachineType, HANDLE hProcess, HANDLE
 
 void PrintStackTrace(CONTEXT* context, HANDLE hLogFile, char* szOutBuffer) {
 	DWORD NumBytes;
-	HMODULE dbgHelp = LoadLibraryA("dbghelp.dll");
-	if (dbgHelp == NULL) {
-		sprintf(szOutBuffer, "   Error: dbghelp.dll not found.\n");
+   char szModuleName[MAX_PATH+1];
+	DWORD dwRes = GetModuleFileNameA(NULL, szModuleName, MAX_PATH);
+	if (dwRes == 0) {
+		sprintf(szOutBuffer, "   Error: GetModuleFileName fails.\n");
 		WriteFile(hLogFile, szOutBuffer, strlen(szOutBuffer), &NumBytes, 0);
 		return;
 	}
+   char* backSlash = strrchr(szModuleName, '\\');
+   if (backSlash != 0) {
+      backSlash[1] = 0;
+   }
+   strcat(szModuleName, "dbghelp.dll");
+	HMODULE dbgHelp = LoadLibraryA(szModuleName);
+	if (dbgHelp == NULL) {
+		sprintf(szOutBuffer, "   Error: %s not found.\n", szModuleName);
+		WriteFile(hLogFile, szOutBuffer, strlen(szOutBuffer), &NumBytes, 0);
+      // try without path
+      dbgHelp = LoadLibraryA("dbghelp.dll");
+      if (dbgHelp == NULL) {
+         sprintf(szOutBuffer, "   Error: dbghelp.dll not found.\n");
+         WriteFile(hLogFile, szOutBuffer, strlen(szOutBuffer), &NumBytes, 0);
+         return;
+      }
+	}
 	pStackWalk stackWalk = (pStackWalk)GetProcAddress(dbgHelp, "StackWalk");
 	if (stackWalk == NULL) {
 		sprintf(szOutBuffer, "   Error: dbghelp.dll does not include StackWalk.\n");
@@ -163,14 +181,14 @@ LONG WINAPI MyUnhandledExceptionFilter(EXCEPTION_POINTERS *info) {
 	}
 	bFirstTime = false;
 	// Get temporary path name
-    TCHAR lpPathBuffer[MAX_PATH+1];
-    DWORD dwRetVal = GetTempPath(MAX_PATH, lpPathBuffer);
-    if (dwRetVal > MAX_PATH || dwRetVal == 0) {
+	TCHAR lpPathBuffer[MAX_PATH+1];
+	DWORD dwRetVal = GetTempPath(MAX_PATH, lpPathBuffer);
+	if (dwRetVal > MAX_PATH || dwRetVal == 0) {
 		return EXCEPTION_CONTINUE_SEARCH;
 	}
 	// Get temporary file name
-    TCHAR szTempName[MAX_PATH+1];
-   	UINT uRetVal = GetTempFileName(lpPathBuffer, TEXT("gle"), 0, szTempName);
+	TCHAR szTempName[MAX_PATH+1];
+	UINT uRetVal = GetTempFileName(lpPathBuffer, TEXT("gle"), 0, szTempName);
 	if (uRetVal = 0) {
 		return EXCEPTION_CONTINUE_SEARCH;
 	}
