@@ -37,6 +37,8 @@
 #include "newfile.h"
 #include "consolewindow.h"
 #include "dialogues.h"
+#include "3dviewer.h"
+#include "../gle/surface/gsurface.h"
 // #include "downloader.h"
 
 #include <iostream>
@@ -609,11 +611,11 @@ void GLEMainWindow::initLibGS()
 	GSLibFunctions* fct = GSLibFunctions::getInstance();
 	// First try the location given in the current settings
 	QString location = settings->getLibGSLocation();
-	if (fct->loadLibrary(location, error) != 0)	{
+	if (fct->loadLibrary(location, error) != 0) {
 		// Next try the location proposed by GLE
 		string gsloc = gleInterface->getGhostScriptLocation();
 		location = QString::fromUtf8(gsloc.c_str());
-		if (fct->loadLibrary(location, error) == 0)	{
+		if (fct->loadLibrary(location, error) == 0) {
 			found = true;
 		}
 	} else {
@@ -704,7 +706,7 @@ void GLEMainWindow::updateWindowTitle()
 	GLEFileInfo* currFile = getCurrentFile();
 	bool isGLE = currFile->isGLE();
 	bool has = isGLE || currFile->isEPS();
-	QString title = QString("%1 %2").arg(APP_NAME).arg(QGLE_VERSION);
+	QString title(APP_NAME);
 	if (has) {
 		title += tr(" - ") + currFile->primaryFile();
 		// Always include "[*]", it is treated special by Qt and only filled in when actually modified
@@ -722,6 +724,8 @@ void GLEMainWindow::updateEnableActions(bool enable)
 	enabledState = enable && has;
 	editorAct->setEnabled(has);
 	copyBitmapAct->setEnabled(has);
+	if (copyPDFAct != NULL) copyPDFAct->setEnabled(has);
+	if (copyEMFAct != NULL) copyEMFAct->setEnabled(has);
 	saveAsAct->setEnabled(has && enable);
 	saveAct->setEnabled(has && drawingArea->isDirty() && enable);
 	exportAct->setEnabled(isGLE && enable);
@@ -736,6 +740,7 @@ void GLEMainWindow::updateEnableActions(bool enable)
 	newAct->setEnabled(enable);
 	evaluatorAct->setEnabled(enable);
 	ghostScriptLogWindowAct->setEnabled(enable);
+	view3DAct->setEnabled(enable && gleInterface->getSurface()->z != NULL);
 	pointerToolAct->setEnabled(enable);
 	moveToolAct->setEnabled(enable);
 	lineToolAct->setEnabled(enable);
@@ -784,6 +789,14 @@ void GLEMainWindow::createActions()
 		connect(copyPDFAct, SIGNAL(triggered()), this, SLOT(copyPDF()));
 	#endif
 
+	//! Action causing the figure to be copied to the clipboard as EMF
+	copyEMFAct = NULL;
+	#ifdef Q_WS_WIN
+		copyEMFAct = new QAction(tr("&Copy as EMF"), this);
+		copyEMFAct->setStatusTip(tr("Copy current diagram as EMF to clipboard"));
+		connect(copyEMFAct, SIGNAL(triggered()), this, SLOT(copyEMF()));
+	#endif
+
 	// Quit the application
 	quitAct = new QAction(tr("&Quit"), this);
 	quitAct->setShortcut(Qt::Key_Q | Qt::ControlModifier);
@@ -811,6 +824,11 @@ void GLEMainWindow::createActions()
 	connect(openInNewWindowAct, SIGNAL(triggered()),
 			this, SLOT(openFileInNewWindow()));
 
+   browseAct = new QAction(tr("&Browse Script Folder"), this);
+   browseAct->setStatusTip(tr("Browse the folder that contains the GLE script"));
+	connect(browseAct, SIGNAL(triggered()),
+			this, SLOT(browseScriptFolder()));
+
 	// Save the file
 	saveAct = new QAction(QIcon(":images/save.png"), tr("&Save"), this);
 	saveAct->setShortcut(Qt::Key_S | Qt::ControlModifier);
@@ -1059,10 +1077,15 @@ void GLEMainWindow::createActions()
 	connect(evaluatorAct, SIGNAL(triggered()),
 			this, SLOT(openEvaluator()));
 
+	// Open 3D view dialogue
+	view3DAct = new QAction(tr("&3D Surface Plot"), this);
+	view3DAct->setStatusTip(tr("Manipulate 3D surface plot"));
+	connect(view3DAct, SIGNAL(triggered()),	this, SLOT(openView3DDialogue()));
+
+	// Open the dialog displaying GhostScript's logs
 	ghostScriptLogWindowAct = new QAction(tr("&GhostScript's Log"), this);
 	ghostScriptLogWindowAct->setStatusTip(tr("Display GhostScript's Error Log"));
-	connect(ghostScriptLogWindowAct, SIGNAL(triggered()),
-			this, SLOT(openGhostScriptLogDialogue()));
+	connect(ghostScriptLogWindowAct, SIGNAL(triggered()), this, SLOT(openGhostScriptLogDialogue()));
 
 	// Test the download function
 	downloadAct = new QAction(tr("&Download"), this);
@@ -1522,25 +1545,35 @@ void GLEMainWindow::renderEPS()
 
 void GLEMainWindow::renderEPS(double dpi)
 {
-	const QString& file = getCurrentFile()->epsFile();
-	QFileInfo fi(file);
-	// If the file is readable, re-render it in the new scale
-	if (fi.isReadable())
-	{
-		renderEPS(file, dpi, getScrollAreaSize());
-	}
-	else
-	{
-		renderComplete(QImage());
-	}
+	renderEPS(getCurrentFile()->epsFile(), dpi, getScrollAreaSize());
 }
 
 // Render an EPS file from a given QString
 void GLEMainWindow::renderEPS(QString epsFile, double dpi, const QSize& area)
 {
 	qDebug() << "About to render " << epsFile;
-	updateEnableActions(false);
-	renderThread->renderEPSToImage(epsFile, dpi, area);
+	if (getCurrentFile()->isGLE())
+	{
+		GLEScript* script = getGLEScript();
+		if (!script->getRecordedBytesBuffer(GLE_DEVICE_EPS)->empty())
+		{
+			updateEnableActions(false);
+			renderThread->renderEPSFromMemoryToImage(script, epsFile, dpi, area);
+			return;
+		}
+	}
+	else
+	{
+		QFileInfo fi(epsFile);
+		// If the file is readable, re-render it in the new scale
+		if (fi.isReadable())
+		{
+			updateEnableActions(false);
+			renderThread->renderEPSFromFileToImage(epsFile, dpi, area);
+			return;
+		}
+	}
+	renderComplete(QImage());
 }
 
 // Refresh GLEDrawing's display
@@ -1618,6 +1651,10 @@ QString GLEMainWindow::getGLEVersion() {
 	return QString::fromUtf8(gleInterface->getGLEVersion().c_str());
 }
 
+QString GLEMainWindow::getGLEBuildDate() {
+	return QString::fromUtf8(gleInterface->getGLEBuildDate().c_str());
+}
+
 const char* GLEMainWindow::getGLETop() {
 	return gleInterface->getGLETop();
 }
@@ -1683,7 +1720,7 @@ void GLEMainWindow::shouldAutoShowConsole()
 // SLOT: display the about box
 void GLEMainWindow::about()
 {
-	box = new AboutBox(this);
+	box = new AboutBox(this, getGLEInterface());
 	box->setModal(true);
 	box->show();
 }
@@ -1750,6 +1787,7 @@ void GLEMainWindow::createMenus()
 	fileMenu->addAction(saveAct);
 	fileMenu->addAction(saveAsAct);
 	fileMenu->addAction(exportAct);
+   fileMenu->addAction(browseAct);
 	// fileMenu->addAction(printAct);
 
 	separatorAct = fileMenu->addSeparator();
@@ -1767,6 +1805,7 @@ void GLEMainWindow::createMenus()
 	QMenu *editMenu = menuBar()->addMenu(tr("&Edit"));
 	editMenu->addAction(copyBitmapAct);
 	if (copyPDFAct != NULL) editMenu->addAction(copyPDFAct);
+	if (copyEMFAct != NULL) editMenu->addAction(copyEMFAct);
 
 	QMenu *viewMenu = menuBar()->addMenu(tr("&View"));
 	viewMenu->addAction(reloadAct);
@@ -1780,6 +1819,7 @@ void GLEMainWindow::createMenus()
 	toolsMenu = menuBar()->addMenu(tr("&Tools"));
 	toolsMenu->addAction(settingsAct);
 	toolsMenu->addAction(evaluatorAct);
+	toolsMenu->addAction(view3DAct);
 	toolsMenu->addAction(ghostScriptLogWindowAct);
 	// toolsMenu->addAction(downloadAct);
 
@@ -1913,6 +1953,13 @@ void GLEMainWindow::openSettingsDialogue()
 	settingsDialogue.exec();
 }
 
+// SLOT: Display 3D viewer
+void GLEMainWindow::openView3DDialogue()
+{
+	QGLE3DViewer* viewer = new QGLE3DViewer(this, gleInterface);
+	viewer->show();
+}
+
 // SLOT: Display GhostScript's log dialogue box
 void GLEMainWindow::openGhostScriptLogDialogue()
 {
@@ -2091,6 +2138,13 @@ void GLEMainWindow::openFileInNewWindow()
 	}
 }
 
+// SLOT: Browse the script folder
+void GLEMainWindow::browseScriptFolder()
+{
+   QFileInfo fname(getCurrentFile()->primaryFile());
+	showLocalFile(fname.dir().path());
+}
+
 // SLOT: Open an existing file
 void GLEMainWindow::openFile(QString fileName, bool isOnStartup)
 {
@@ -2388,7 +2442,7 @@ void GLEMainWindow::copyBitmap() {
 bool copyPDFMac(const string& fname);
 #endif
 
-//! Action causing the current figure to be copied to the clipboard	as PDF
+//! Action causing the current figure to be copied to the clipboard as PDF
 void GLEMainWindow::copyPDF() {
 	QString tempDir = QDir::tempPath();
 	QRegExp rxEndSlash(".*[\\\\/]$");
@@ -2418,6 +2472,17 @@ void GLEMainWindow::copyPDF() {
 	#endif
 }
 
+//! Action causing the current figure to be copied to the clipboard as EMF
+void GLEMainWindow::copyEMF() {
+	GLEInterface* iface = getGLEInterface();
+	clearConsoleWindow();
+	iface->clearAllCmdLine();
+	iface->setMakeDrawObjects(false);
+	iface->setCmdLineOption("copy");
+	iface->renderGLE(getGLEScript(), "", GLE_DEVICE_EMF);
+	iface->clearAllCmdLine();
+}
+
 void GLEMainWindow::resetDrawing()
 {
 	// Reset the drawing.
