@@ -61,6 +61,7 @@ public:
 
 	//! Return the GLE version
 	QString getGLEVersion();
+	QString getGLEBuildDate();
 	//! Return the GLE version
 	const char* getGLETop();
 	//! Return the current compatibility mode
@@ -123,6 +124,8 @@ private slots:
 	void openSettingsDialogue();
 	//! Display GhostScript's log dialogue box
 	void openGhostScriptLogDialogue();
+	//! Display 3D viewer
+	void openView3DDialogue();
 	//! Display the expression evaluator dialogue box
 	void openEvaluator();
 	//! Display export dialogue box
@@ -133,6 +136,8 @@ private slots:
 	void openDownloadDialogue();
 	//! Open file in new window
 	void openFileInNewWindow();
+   //! Browse the folder containing the current script
+   void browseScriptFolder();
 	//! Open an existing file
 	void openFile(QString fileName = QString(), bool isOnStartup = false);
 	//! Open a recently opened file
@@ -153,8 +158,10 @@ private slots:
 	void save(QString fileName);
 	//! Action causing the current bitmap to be copied to the clipboard
 	void copyBitmap();
-	//! Action causing the current figure to be copied to the clipboard	as PDF
+	//! Action causing the current figure to be copied to the clipboard as PDF
 	void copyPDF();
+	//! Action causing the current figure to be copied to the clipboard as EMF
+	void copyEMF();
 	//! Switch to preview mode
 	void previewModeToggle(bool state);
 	//! Switch to edit mode
@@ -304,6 +311,8 @@ private:
 	QAction *copyBitmapAct;
 	//! Action causing the figure to be copied to the clipboard as PDF
 	QAction *copyPDFAct;
+	//! Action causing the figure to be copied to the clipboard as EMF
+	QAction *copyEMFAct;
 	//! Linked to the slot: about()
 	QAction *aboutAct;
 	//! Open GLE website
@@ -328,12 +337,16 @@ private:
 	QAction *settingsAct;
 	//! Display expression evaluator
 	QAction *evaluatorAct;
+	//! View 3D part of script
+	QAction *view3DAct;
 	//! Display GhostScript's log window
 	QAction *ghostScriptLogWindowAct;
 	//! Open an existing file
 	QAction *openInNewWindowAct;
 	//! Open an existing file
 	QAction *openAct;
+   //! Browse the script folder
+   QAction *browseAct;
 	//! Save the file under a new name
 	QAction *saveAsAct;
 	//! Save the file
