@@ -225,11 +225,27 @@ void GLERenderThread::renderGLEToImage(GLEScript* script, const QString& epsFile
 	}
 }
 
-//! Convert an EPS file to an image (dpi == 0 -> autoscale)
-void GLERenderThread::renderEPSToImage(const QString& fname, double dpi, const QSize& area)
+void GLERenderThread::renderEPSFromMemoryToImage(GLEScript* script, const QString& epsFile, double dpi, const QSize& area)
 {
 	QMutexLocker locker(&mutex);
-	todo = ToDoRenderEPS;
+	todo = ToDoRenderEPSFromMemory;
+	gleScript = script;
+	epsFileName = epsFile;
+	epsDPI = dpi;
+	epsArea = area;
+	if (!isRunning()) {
+		start();
+	} else {
+		restart = true;
+		condition.wakeOne();
+	}
+}
+
+//! Convert an EPS file to an image (dpi == 0 -> autoscale)
+void GLERenderThread::renderEPSFromFileToImage(const QString& fname, double dpi, const QSize& area)
+{
+	QMutexLocker locker(&mutex);
+	todo = ToDoRenderEPSFromFile;
 	epsFileName = fname;
 	epsDPI = dpi;
 	epsArea = area;
@@ -243,54 +259,58 @@ void GLERenderThread::renderEPSToImage(const QString& fname, double dpi, const Q
 
 void GLERenderThread::run()
 {
+	qRegisterMetaType<QImage>("QImage");
 	forever
 	{
-	mutex.lock();
-	if (abort)
-	{
+		mutex.lock();
+		if (abort)
+		{
+			mutex.unlock();
+			return;
+		}
+		int myToDo = todo;
+		QString myEpsFileName = epsFileName;
+		double myEpsDPI = epsDPI;
+		QSize myEpsArea = epsArea;
+		GLERC<GLEScript> myGLEScript;
+		if (myToDo == ToDoRenderGLE || myToDo == ToDoRenderEPSFromMemory) {
+			// script is NULL if render EPS
+			myGLEScript = gleScript;
+		}
 		mutex.unlock();
-		return;
-	}
-	int myToDo = todo;
-	QString myEpsFileName = epsFileName;
-	double myEpsDPI = epsDPI;
-	QSize myEpsArea = epsArea;
-	GLERC<GLEScript> myGLEScript;
-	if (myToDo == ToDoRenderGLE) {
-		// script is NULL if render EPS
-		myGLEScript = gleScript;
-	}
-	mutex.unlock();
 
-	// do processing
-	if (myToDo == ToDoRenderGLE)
-	{
-		emit serverMessage(tr("Rendering GLE file"));
+		// do processing
+		if (myToDo == ToDoRenderGLE)
+		{
+			emit serverMessage(tr("Rendering GLE file"));
 			GLEInterface* iface = myGLEScript->getGLEInterface();
-			iface->renderGLE(myGLEScript.get(), myEpsFileName.toLatin1().constData(), GLE_DEVICE_EPS);
+			iface->renderGLE(myGLEScript.get(), myEpsFileName.toLatin1().constData(), GLE_DEVICE_EPS, true);
 			if (iface->getOutput()->getExitCode() == 0)
 			{
 				// Successful run, now render EPS
-				myToDo = ToDoRenderEPS;
+				myToDo = ToDoRenderEPSFromMemory;
 			}
 			else
 			{
 				QImage emptyImage;
 				emit renderComplete(emptyImage);
 			}
-	}
-	if (myToDo == ToDoRenderEPS)
-	{
-		renderEPSToImageInternal(myEpsFileName, myEpsDPI, myEpsArea);
-	}
-
-	mutex.lock();
-	if (!restart)
-	{
-		 condition.wait(&mutex);
-	}
-	restart = false;
-	mutex.unlock();
+		}
+		if (myToDo == ToDoRenderEPSFromMemory)
+		{
+			renderEPSToImageInternalFromMemory(myGLEScript.get(), myEpsDPI, myEpsArea);
+		}
+		if (myToDo == ToDoRenderEPSFromFile)
+		{
+			renderEPSToImageInternalFromFile(myEpsFileName, myEpsDPI, myEpsArea);
+		}
+		mutex.lock();
+		if (!restart)
+		{
+			 condition.wait(&mutex);
+		}
+		restart = false;
+		mutex.unlock();
 	}
 }
 
@@ -299,8 +319,55 @@ QString& GLERenderThread::getGhostScriptOutput()
 	return GhostScriptOutput;
 }
 
+GSInterpreterLib* GLERenderThread::setupInterpreter(const QSize& area, double b1, double b2, double width, double height, double& dpi, bool& newDPI) {
+	// Auto scale new files to size of drawing area
+	if (dpi == 0.0) {
+		dpi = QGLE::computeAutoScaleDPIFromPts(area, 5, width, height);
+		newDPI = true;
+	}
+	// Create the interpreter object and set a few parameters
+	GSInterpreterLib *interpreter = new GSInterpreterLib();
+	interpreter->setOrigin(b1, b2);
+	interpreter->setMedia("a4");
+	interpreter->setMagnify(1);
+	interpreter->setOrientation(0);
+	interpreter->setDPI(dpi);
+	int img_wd = GLEBBoxToPixels(dpi, width);
+	int img_hi = GLEBBoxToPixels(dpi, height);
+	interpreter->setSize(img_wd, img_hi);
+	interpreter->setAABits(4,4);
+	return interpreter;
+}
+
+void GLERenderThread::renderEPSDone(GSInterpreterLib* interpreter, double dpi, bool newDPI) {
+	QString gsError;
+	GhostScriptOutput = interpreter->getOutpuStreamText();
+	if (interpreter->hasError()) {
+		const GSError& e = interpreter->getError();
+		gsError = QString("Ghostscript error: %1 %2").arg(e.getCode()).arg(e.getName());
+		qDebug() << "Error: " << gsError;
+	}
+
+	QImage image(interpreter->getImage());
+	delete interpreter;
+
+	// Delete interpreter should go before different signals
+	// otherwise recursive calls to this function may cause multiple instantiations of
+	// libgs, which is not allowed and results in crashes
+	if (gsError != "")
+	{
+		emit serverError(gsError);
+	}
+	if (newDPI)
+	{
+		emit dpiChanged(dpi);
+	}
+	emit renderComplete(image);
+
+}
+
 //! Convert an EPS file to an image (dpi == 0 -> autoscale)
-void GLERenderThread::renderEPSToImageInternal(const QString& fname, double dpi, const QSize& area)
+void GLERenderThread::renderEPSToImageInternalFromFile(const QString& fname, double dpi, const QSize& area)
 {
 	// Check that the file can be opened
 	QFile file(fname);
@@ -355,55 +422,29 @@ void GLERenderThread::renderEPSToImageInternal(const QString& fname, double dpi,
 		}
 	}
 
-	// Auto scale new files to size of drawing area
 	bool newDPI = false;
-	if (dpi == 0.0) {
-		dpi = QGLE::computeAutoScaleDPIFromPts(area, 5, width, height);
-		newDPI = true;
-	}
-
-	// Create the interpreter object and set a few parameters
-	GSInterpreterLib *interpreter = new GSInterpreterLib();
-	interpreter->setOrigin(b1, b2);
-	interpreter->setMedia("a4");
-	interpreter->setMagnify(1);
-	interpreter->setOrientation(0);
-	interpreter->setDPI(dpi);
-	int img_wd = GLEBBoxToPixels(dpi, width);
-	int img_hi = GLEBBoxToPixels(dpi, height);
-	interpreter->setSize(img_wd, img_hi);
-	interpreter->setAABits(4,4);
-
-	qRegisterMetaType<QImage>("QImage");
+	GSInterpreterLib* interpreter = setupInterpreter(area, b1, b2, width, height, dpi, newDPI);
 
 	// Run the interpreter
 	file.reset();
 	interpreter->run(file);
 	file.close();
 
-	QString gsError;
-	GhostScriptOutput = interpreter->getOutpuStreamText();
-	if (interpreter->hasError()) {
-		const GSError& e = interpreter->getError();
-		gsError = QString("Ghostscript error: %1 %2").arg(e.getCode()).arg(e.getName());
-		qDebug() << "Error: " << gsError;
-	}
+	renderEPSDone(interpreter, dpi, newDPI);
+}
 
-	QImage image = interpreter->getImage();
-	delete interpreter;
+void GLERenderThread::renderEPSToImageInternalFromMemory(GLEScript* script, double dpi, const QSize& area) {
+	// Rendering EPS has started
+	emit serverMessage(tr("Rendering PostScript"));
 
-	// Delete interpreter should go before different signals
-	// otherwise recursive calls to this function may cause multiple instantiations of
-	// libgs, which is not allowed and results in crashes
-	if (gsError != "")
-	{
-		emit serverError(gsError);
-	}
-	if (newDPI)
-	{
-		emit dpiChanged(dpi);
-	}
-	emit renderComplete(image);
+	// Render EPS from memory
+	GLEPoint bb(script->getBoundingBox());
+	GLEPoint origin(script->getBoundingBoxOrigin());
+	bool newDPI = false;
+	GSInterpreterLib* interpreter = setupInterpreter(area, origin.getX(), origin.getY(), bb.getX(), bb.getY(), dpi, newDPI);
+	string* bytes = script->getRecordedBytesBuffer(GLE_DEVICE_EPS);
+	interpreter->run(bytes);
+	renderEPSDone(interpreter, dpi, newDPI);
 }
 
 void GLERenderThread::startRender(const GLERectangle& rect, double dpi)
