@@ -92,7 +92,8 @@ public:
 	~GLERenderThread();
 
 	enum {
-		ToDoRenderEPS,
+		ToDoRenderEPSFromFile,
+		ToDoRenderEPSFromMemory,
 		ToDoRenderGLE
 	};
 
@@ -100,7 +101,8 @@ public:
 	void renderGLEToImage(GLEScript* script, const QString& epsFile, double dpi, const QSize& area);
 
 	//! Convert an EPS file to an image (dpi == 0 -> autoscale)
-	void renderEPSToImage(const QString& fname, double dpi, const QSize& area);
+	void renderEPSFromMemoryToImage(GLEScript* script, const QString& epsFile, double dpi, const QSize& area);
+	void renderEPSFromFileToImage(const QString& fname, double dpi, const QSize& area);
 
 	//! Return GhostScript's output stream
 	QString& getGhostScriptOutput();
@@ -125,7 +127,12 @@ protected:
 	void run();
 
 	//! Convert an EPS file to an image (dpi == 0 -> autoscale)
-	void renderEPSToImageInternal(const QString& fname, double dpi, const QSize& area);
+	void renderEPSToImageInternalFromFile(const QString& fname, double dpi, const QSize& area);
+
+	void renderEPSToImageInternalFromMemory(GLEScript* script, double dpi, const QSize& area);
+
+	GSInterpreterLib* setupInterpreter(const QSize& area, double b1, double b2, double width, double height, double& dpi, bool& newDPI);
+	void renderEPSDone(GSInterpreterLib* interpreter, double dpi, bool newDPI);
 
 protected:
 	//! Set true to quit
