@@ -115,7 +115,7 @@ void SettingsDialogue::initDrawingTab()
 void SettingsDialogue::initToolTab()
 {
 	// Set up the tab based on the application's current settings
-	toolTab->setToolGLE(mainWin->getGLEVersion(), mainWin->getGLETop());
+	toolTab->setToolGLE(mainWin->getGLEVersion(), mainWin->getGLEBuildDate(), mainWin->getGLETop());
 	toolTab->setToolGsLib(mainWin->settings->getLibGSLocation(), mainWin->getGsLibVersion());
 	toolTab->setToolEditor(mainWin->settings->editorLocation());
 }
@@ -177,7 +177,7 @@ ApplicationTab::ApplicationTab(QWidget *parent)
 	autoScaleOnOpenBox = new QCheckBox(tr("Auto scale when opening diagrams"));
 	askAboutObjectsBox = new QCheckBox(tr("Automatically keep objects when reloading"));
 
-	QLabel *resolutionLabel = new QLabel(tr("Default Resolution:"));
+	QLabel *resolutionLabel = new QLabel(tr("Default resolution:"));
 	resolutionBox = new QSpinBox();
 	resolutionBox->setMinimum(0);
 	resolutionBox->setMaximum(500);
@@ -337,7 +337,7 @@ bool ApplicationTab::askAboutObjects()
 ServerTab::ServerTab(QWidget *parent)
 	: QWidget(parent)
 {
-	QLabel *portNumLabel = new QLabel(tr("Server Port Number:"));
+	QLabel *portNumLabel = new QLabel(tr("Server port number:"));
 	portNumBox = new QSpinBox();
 	portNumBox->setMaximum(65535);
 	portNumBox->setMinimum(1);
@@ -403,7 +403,7 @@ DrawingTab::DrawingTab(QWidget *parent)
 	yBox->setSingleStep(0.1);
 
 
-	equalSpacing = new QCheckBox("Equal Spacing");
+	equalSpacing = new QCheckBox("Equal spacing");
 	connect(equalSpacing, SIGNAL(toggled(bool)),
 			this, SLOT(equalSpacingToggled(bool)));
 
@@ -423,8 +423,8 @@ DrawingTab::DrawingTab(QWidget *parent)
 	orthoSnapOnStartBox = new QCheckBox(tr("Start with orthogonal snap enabled"));
 	polarSnapOnStartBox = new QCheckBox(tr("Start with polar snap enabled"));
 	polarTrackOnStartBox = new QCheckBox(tr("Start with polar tracking enabled"));
-	QLabel *startLabel = new QLabel(tr("Polar Snap Start Angle:"));
-	QLabel *incLabel = new QLabel(tr("Polar Snap Increment Angle:"));
+	QLabel *startLabel = new QLabel(tr("Polar snap start angle:"));
+	QLabel *incLabel = new QLabel(tr("Polar snap increment angle:"));
 	polarSnapStartAngleBox = new QDoubleSpinBox();
 	polarSnapStartAngleBox->setMaximum(360.0);
 	polarSnapStartAngleBox->setMinimum(0.0);
@@ -595,6 +595,7 @@ ToolTab::ToolTab(QWidget *parent, GLEMainWindow *main) : QWidget(parent)
 	ToolGsLib = new QLineEdit();
 	ToolEditor = new QLineEdit();
 	GLEVersion = new QLabel();
+	GLEBuildDate = new QLabel();
 	GLETop = new QLineEdit();
 	GsLibVersion = new QLabel();
 
@@ -622,11 +623,12 @@ ToolTab::ToolTab(QWidget *parent, GLEMainWindow *main) : QWidget(parent)
 
 	QVBoxLayout *layout = new QVBoxLayout();
 	layout->addWidget(GLEVersion);
+	layout->addWidget(GLEBuildDate);
 	layout->addWidget(new QLabel(tr("GLE_TOP:")));
 	layout->addLayout(gle);
 	layout->addWidget(GsLibVersion);
 	layout->addLayout(gs);
-	layout->addWidget(new QLabel(tr("Text Editor")));
+	layout->addWidget(new QLabel(tr("Text editor")));
 	layout->addLayout(editor);
 	layout->addStretch(1);
 
@@ -677,9 +679,10 @@ void ToolTab::browseToolEditor() {
 	if (!location.isEmpty()) ToolEditor->setText(location);
 }
 
-void ToolTab::setToolGLE(const QString& vers, const QString& top)
+void ToolTab::setToolGLE(const QString& vers, const QString& date, const QString& top)
 {
-	GLEVersion->setText(QString("GLE Version: ") + vers);
+	GLEVersion->setText(QString("GLE version: ") + vers);
+	GLEBuildDate->setText(QString("GLE build date: ") + date);
 	GLETop->setText(top);
 }
 
