@@ -438,12 +438,11 @@ static BOOL parseGlobals(FILE *fp, GlobalFontInfo *gfi)
  *  file is reset to be where it was upon entering this function.
  */
 
-// ** GLE ** commented out unused method initializeArray(fp, cwi)
-
-/*
-static int initializeArray(fp, cwi)
-  FILE *fp;
-  register int *cwi;
+//static int initializeArray(fp, cwi)
+//  FILE *fp;
+//  register int *cwi;
+// ** GLE ** replaced above three lines by the following one
+int initializeArray(FILE *fp, int *cwi)
 {
     BOOL cont = TRUE, found = FALSE;
     long opos = ftell(fp);
@@ -456,7 +455,7 @@ static int initializeArray(fp, cwi)
         if (keyword == NULL)
         {
             error = earlyEOF;
-            break; // get out of loop
+            break; /* get out of loop */
         }
         switch(recognize(keyword))
         {
@@ -488,8 +487,8 @@ static int initializeArray(fp, cwi)
             default:
                 error = parseError;
                 break;
-        } // switch
-    } // while
+        } /* switch */
+    } /* while */
 
     if (!found)
         width = 250;
@@ -501,8 +500,8 @@ static int initializeArray(fp, cwi)
 
     return(error);
 
-} // initializeArray
-*/
+} /* initializeArray */
+
 
 /************************* parseCharWidths **************************/
 
